@@ -11,8 +11,13 @@ while (<stdin>) {
   }
   if ($state == 2) {
     if ($_ =~ /MEME version (\d\.\d\.\d) \((\w+ (\d+, |)\d\d\d\d)\)/ ) {
+      # RELEASE line
       print "<p><hr></p>\n";
-      print "* [$2] Release $1 MEME Suite\n";
+      print "* <b>[$2] Release $1 MEME Suite</b>\n";
+    } elsif ($_ =~ /<b>\[\w+ (\d|\d\d), \d\d\d\d\]<\/b>/) {
+      # NEWS line
+      $_ =~ s/^\s+//g;
+      print "* $_";
     } else {
       print;
     }
