@@ -12,6 +12,9 @@ function $(el) {
 */
 var new_icon_src = "data:image/png;base64,iVBORw0KGgoAAAANSUhEUgAAABwAAAAQCAMAAAAyEe/dAAAACVBMVEX//wAAAAD////s2cV/AAAAdUlEQVQYlXVRBxLAIAhL+P+jC2HZhXcBZEWEldDsZcLIcAhHWWnK8SDcWQhMFUHdAQ1CqQ5+CWPmlHojl+nCJNRtzu4qRc3IUzmTVpXYK0nox0z0PI1stgchdK7lEv7ekhvalw8WW547Gyzedt/2/gLx8WXjXF/1AYFriNAWAAAAAElFTkSuQmCC";
 
+// Forbidden file extensions
+const forbidden = new Set(["jsp", "jspx", "war", "exe", "bat", "sh", "php", "pl", "png", "py", "dll", "class", "xml"]);
+
 /*
  * See http://stackoverflow.com/a/5450113/66387
  * Does string multiplication like the perl x operator.
