@@ -15,20 +15,7 @@ if (!spamo_alphabet.has_complement()) {
   orient_desc = ["upstream", "downstream"];
 }
 
-var DelayLogoTask = function(logo, canvas) {
-  "use strict";
-  canvas.width = canvas.width; // clear canvas
-  this.logo = logo;
-  this.canvas = canvas;
-};
-
-DelayLogoTask.prototype.run = function () {
-  "use strict";
-  this.canvas.width = this.canvas.width; // clear canvas
-  draw_logo_on_canvas(this.logo, this.canvas);
-};
-
-function make_logo(canvas, motif) {
+function make_logo_spamo(canvas, motif) {
   var pspm;
   pspm = new Pspm(motif.pwm, motif.id, motif.ltrim, motif.rtrim, motif.nsites);
   var logo = new Logo(spamo_alphabet);
@@ -339,7 +326,7 @@ function make_pri_table() {
     row = pri_row.cloneNode(true);
     row.querySelector(".pri_db").appendChild(document.createTextNode(data.primary_dbs[primary.db].name));
     row.querySelector(".pri_name").appendChild(make_name(primary, false, true));
-    make_logo(row.querySelector(".pri_logo"), primary);
+    make_logo_spamo(row.querySelector(".pri_logo"), primary);
     row.querySelector(".pri_nlist").appendChild(document.createTextNode(secondaries.length));
     row_mlist = row.querySelector(".pri_list");
     for (j = 0; j < secondaries.length; j++) {
@@ -490,7 +477,6 @@ function download_all_contr_seqs(type) {
   prepare_download(text, mimetype, "spamo_contr_seqs." + type);
 }
 
-//FIXME_TLB
 function isArray(x) {
     return x.constructor.toString().indexOf("Array") > -1;
 }
@@ -585,7 +571,7 @@ function update_selected_secondary(manager) {
   _set_text(manager.block.querySelector(".sa_gap"), best_sp.bin);
   _set_text(manager.block.querySelector(".sa_orient"), orient_desc[best_sp.orient]);
   // fill in the secondary logo
-  make_logo(manager.block.querySelector(".sa_slogo_in"), manager.secondary.motif);
+  make_logo_spamo(manager.block.querySelector(".sa_slogo_in"), manager.secondary.motif);
   // get the spacings table
   table = manager.block.querySelector(".sa_spacings");
   // clear the table
@@ -1028,7 +1014,7 @@ function make_spacing_analysis() {
     // set the name
     block.querySelector(".sa_primary_name").appendChild(make_name(primary.motif));
     // make the logo
-    make_logo(block.querySelector(".sa_plogo_in"), primary.motif);
+    make_logo_spamo(block.querySelector(".sa_plogo_in"), primary.motif);
     // setup options
     setup_filter_sort(manager, i);
     setup_highlight(manager, i);
