@@ -339,6 +339,13 @@ function action_outpop_submit(e) {
   input.setAttribute("value", motifs);
   form.appendChild(input);
 
+  var current_file = location.pathname.substring(location.pathname.lastIndexOf('/')+1);
+  input = document.createElement("input");
+  input.setAttribute("type", "hidden");
+  input.setAttribute("name", "motifs_name");
+  input.setAttribute("value", "motif number " + (current_motif+1) + " from " + current_file);
+  form.appendChild(input);
+
   document.body.appendChild(form);
   form.submit();
   document.body.removeChild(form);
