@@ -14,12 +14,11 @@
 #include "string-list.h"
 #include "object-list.h"
 #include "clustalw-io.h"
+#include "macros.h"	// for myfree
 
 #define MAX_ALIGNMENT_NAME 40     // Longest alignment ID.
 #define MAX_ALIGNMENT_COMMENT 128 // Longest comment.
 
-extern char alphabet[];
-
 // Instantiate the ALIGNMENT_T type.
 struct alignment {
   char  name[MAX_ALIGNMENT_NAME + 1];     // Alignment program name.
@@ -62,12 +61,16 @@ ALIGNMENT_T* allocate_alignment(
       fprintf(stderr, "Warning: truncating alignment program name %s to %s.\n",
 	      name, new_alignment->name);
     }
+  } else {
+    new_alignment->name[0] = '\0';
   }
 
   // Store the description, truncating if necessary.
   if (description != NULL) {
     strncpy(new_alignment->desc, description, MAX_ALIGNMENT_COMMENT);
     new_alignment->desc[MAX_ALIGNMENT_COMMENT] = '\0';
+  } else {
+    new_alignment->desc[0] = '\0';
   }
 
   // Store the sequences.
@@ -79,7 +82,7 @@ ALIGNMENT_T* allocate_alignment(
   int seq_length = strlen(get_raw_sequence(sequences[0]));
   int i;
   for (i = 0; i < num_sequences; i++) {
-    myassert(TRUE,
+    myassert(true,
 	     strlen(get_raw_sequence(sequences[i])) == seq_length,
 	     "Sequence #1 (%s) is length=%d, but sequence #%d (%s) is length=%d.\n<%s>\n",
 	     get_seq_name(sequences[0]), seq_length, i, 
@@ -329,31 +332,25 @@ void get_alignment_col(int col, char* alignment_col, ALIGNMENT_T* alignment) {
  * consecutive n-digit base m numerals: 
  *   AAAA = 0000, AAAC = 0001, ..., TTTG = 3332, TTTT = 3333.
  *************************************************************************/
-int hash_alignment_col(char* alignment_col, int alignment_col_size) {
+int hash_alignment_col(ALPH_T* alph, char* alignment_col, int alignment_col_size) {
 
   // The alignment column must have at least
   // one character aside from the terminating null.
   assert(alignment_col_size >= 1);
   assert(alignment_col != NULL);
 
-  char* alphabet = get_alphabet(FALSE);
-  int alph_size = get_alph_size(ALPH_SIZE);
+  int asize = alph_size_core(alph);
   int hash = 0;
   int place = 1;
-  int i;
+  int i, j;
   for (i = alignment_col_size - 1; i >= 0; i--) {
     // Find alignment column character in alphabet
-    int j;
-    for (j = 0; j < alph_size; j++) {
-      if (alignment_col[i] == alphabet[j]) {
-        break;
-      }
-    }
-    if (j == alph_size)  {
+    j = alph_index(alph, alignment_col[i]);
+    if (j == -1 || j >= asize)  {
       die("Alignment character %c not found in alphabet\n", alignment_col[i]);
     }
     hash += j * place;
-    place *= alph_size;
+    place *= asize;
   }
 
   return hash;
@@ -371,17 +368,17 @@ int hash_alignment_col(char* alignment_col, int alignment_col_size) {
  * for the terminating null.
  *************************************************************************/
 void unhash_alignment_col(
+  ALPH_T* alph,
   int alignment_col_index, 
   char *alignment_col, 
   int alignment_col_size
 ) {
-  char* alphabet = get_alphabet(FALSE);
-  int alph_size = get_alph_size(ALPH_SIZE);
+  int asize = alph_size_core(alph);
 
   assert(alignment_col_index >= 0);
   assert(
     alignment_col_index < pow(
-      (double) alph_size, 
+      (double) asize, 
       (double) alignment_col_index
     )
   );
@@ -391,10 +388,10 @@ void unhash_alignment_col(
   alignment_col[alignment_col_size] = '\0';
   int i, j;
   for (i = alignment_col_size - 1; i >= 0; i--) {
-    j = alignment_col_index % alph_size;
+    j = alignment_col_index % asize;
     alignment_col_index -= j;
-    alignment_col[i] = alphabet[j];
-    alignment_col_index /= alph_size;
+    alignment_col[i] = alph_char(alph, j);
+    alignment_col_index /= asize;
   }
 } // unhash_alignment_col
 
@@ -407,6 +404,7 @@ void unhash_alignment_col(
  *    [.-nNxX]
  *************************************************************************/
 static ARRAY_T* build_alignment_column_counts(
+  ALPH_T* alph,
   ALIGNMENT_T* alignment,
   ARRAY_T* counts 
 ) 
@@ -414,12 +412,12 @@ static ARRAY_T* build_alignment_column_counts(
 
   assert(alignment != NULL);
 
-  int alph_size = get_alph_size(ALPH_SIZE);
+  int asize = alph_size_core(alph);
 
   // Calculate number of possible alignment columns
   // and create storage for counting occurences.
   int num_seqs = get_num_aligned_sequences(alignment);
-  int num_alignment_cols = (int) pow((double) alph_size, (double) num_seqs);
+  int num_alignment_cols = (int) pow((double) asize, (double) num_seqs);
   if (counts == NULL) {
     counts = allocate_array(num_alignment_cols);
   }
@@ -438,7 +436,7 @@ static ARRAY_T* build_alignment_column_counts(
     if (strchr(alignment_col, 'n') != NULL) { continue; }
     if (strchr(alignment_col, 'X') != NULL) { continue; }
     if (strchr(alignment_col, 'x') != NULL) { continue; }
-    h = hash_alignment_col(alignment_col, num_seqs);
+    h = hash_alignment_col(alph, alignment_col, num_seqs);
     incr_array_item(h, 1, counts);
   }
 
@@ -457,8 +455,9 @@ static ARRAY_T* build_alignment_column_counts(
  *  occurs in some alignment.
  ****************************************************************************/
 OBJECT_LIST_T* get_alignment_column_freqs_list
-  (STRING_LIST_T* filenames,
-  BOOLEAN_T remove_allgap_seqs) 
+  (ALPH_T* alph, 
+   STRING_LIST_T* filenames,
+  bool remove_allgap_seqs) 
 {
   int file_index;
   int num_filenames = get_num_strings(filenames);
@@ -484,7 +483,7 @@ OBJECT_LIST_T* get_alignment_column_freqs_list
     // Read the alignment
     int ref_seq_index = 0;
     ALIGNMENT_T* alignment = 
-      read_alignment_from_file(filename, TRUE, remove_allgap_seqs, &ref_seq_index);
+      read_alignment_from_file(filename, true, remove_allgap_seqs, &ref_seq_index);
     STRING_LIST_T* alignment_species = get_species_names(alignment);
 
     // Try to retrieve the counts so far for this list of species.
@@ -497,13 +496,13 @@ OBJECT_LIST_T* get_alignment_column_freqs_list
     // Found counts for current species list?
     if (alignment_column_freqs) {
       // Add counts from current alignment.
-      (void) build_alignment_column_counts(alignment, alignment_column_freqs);
+      (void) build_alignment_column_counts(alph, alignment, alignment_column_freqs);
       // Note: objects in lists are references, so no need to re-store
       // after modification.
     } 
     // Didn't find counts for this species list, so create new array of counts.
     else {
-      alignment_column_freqs = build_alignment_column_counts(alignment, NULL);
+      alignment_column_freqs = build_alignment_column_counts(alph, alignment, NULL);
       store_object(
 	(void*)alignment_column_freqs,
 	(void*)alignment_species,
@@ -537,10 +536,11 @@ OBJECT_LIST_T* get_alignment_column_freqs_list
       set_array_item(i, f/total_counts, alignment_column_freqs);
 
 #ifdef DEBUG
-      int alph_size = get_alph_size(ALPH_SIZE);
-      int num_leaves = NINT(log(num_freqs)/log(alph_size));
+      int asize = alph_size_core(alph);
+      int num_leaves = NINT(log(num_freqs)/log(asize));
       char* alignment_col = mm_malloc((num_leaves + 1) * sizeof(char));
       unhash_alignment_col(
+        alph,
         i, 				//col_index
 	alignment_col,
 	num_leaves
@@ -560,10 +560,10 @@ OBJECT_LIST_T* get_alignment_column_freqs_list
 *  ANY_BASE are not counted. The freq. of ANY_BASE characters is stored
 *  in the last element of the array.
 ****************************************************************************/
-ARRAY_T* get_alignment_freqs(ALIGNMENT_T* alignment) {
+ARRAY_T* get_alignment_freqs(ALPH_T* alph, ALIGNMENT_T* alignment) {
   char c = 0;
-  int alph_index = 0;
-  int alph_size = 0;
+  int aindex = 0;
+  int asize = 0;
   int i = 0;
   int s = 0;
   int total_bases = 0;
@@ -571,28 +571,26 @@ ARRAY_T* get_alignment_freqs(ALIGNMENT_T* alignment) {
   ARRAY_T* freqs = NULL;
   
   // Initialize counts for each character in the alphabet
-  alph_size = get_alph_size(ALPH_SIZE);
-  num_bases = mm_malloc(alph_size * sizeof(int));
-  for (i = 0; i < alph_size; i++) {
+  asize = alph_size_core(alph);
+  num_bases = mm_malloc(asize * sizeof(int));
+  for (i = 0; i < asize; i++) {
     num_bases[i] = 0;
   }
 
   for (s = 0; s < alignment->num_sequences; s++) {
     for (i = 0; i < alignment->length; i++) {
       c = get_seq_char(i, alignment->sequences[s]);
-      if (c != '-' && c != '.') {
-        alph_index = alphabet_index(c, alphabet);
-        // c might be an ambiguity code. We don't count ambiguity codes.
-        if (alph_index < alph_size) {
-          num_bases[alph_index]++;
-          total_bases++;
-        } 
+      aindex = alph_index(alph, c);
+      // c might be an ambiguity code. We don't count ambiguity codes.
+      if (aindex != -1 && aindex < asize) {
+        num_bases[aindex]++;
+        total_bases++;
       }
     }
   }
 
-  freqs = allocate_array(alph_size);
-  for (i = 0; i < alph_size; i++) {
+  freqs = allocate_array(asize);
+  for (i = 0; i < asize; i++) {
     set_array_item(i, (double) num_bases[i] / (double) total_bases, freqs);
   }
 
@@ -602,6 +600,37 @@ ARRAY_T* get_alignment_freqs(ALIGNMENT_T* alignment) {
   return freqs;
 }
 
+/****************************************************************************
+*  Return a frequency matrix for an alignment.
+*  Gaps and ambiguity characters other then
+*  ANY_BASE are not counted.
+****************************************************************************/
+MATRIX_T* get_freq_matrix_from_alignment(ALPH_T* alph, ALIGNMENT_T* alignment) {
+  int i, s;
+  
+  // Allocate the matrix and initialize to zero.
+  int asize = alph_size_core(alph);
+  MATRIX_T *freqs = allocate_matrix(alignment->length, asize);
+  init_matrix(0, freqs);
+
+  // Fill the matrix.
+  for (s = 0; s < alignment->num_sequences; s++) {
+    for (i = 0; i < alignment->length; i++) {
+      char c = get_seq_char(i, alignment->sequences[s]);
+      int aindex = alph_index(alph, c);
+      // c might be an ambiguity code. We don't count ambiguity codes.
+      if (aindex != -1 && aindex < asize) {
+        incr_matrix_cell(i, aindex, 1, freqs);
+      }
+    }
+  }
+
+  // Convert counts to frequencies;
+  normalize_rows(0.0, freqs);
+
+  return freqs;
+}
+
 /****************************************************************************
  *  Get a cumulative count of gaps within one sequence of the alignment
  ****************************************************************************/
@@ -632,31 +661,29 @@ int* get_cumulative_gap_count(int seqIndex, ALIGNMENT_T* alignment) {
 /****************************************************************************
  *  Does a column of an alignment contain gaps?
  ****************************************************************************/
-BOOLEAN_T alignment_site_has_gaps(int index, ALIGNMENT_T* alignment) {
+bool alignment_site_has_gaps(int index, ALIGNMENT_T* alignment) {
 
   int i = 0;
   for (i = 0; i < alignment->num_sequences; i++) {
     char c = get_seq_char(index, alignment->sequences[i]);
     if (c == '-' || c == '.') {
-      return(TRUE);
+      return(true);
     }
   }
-  return(FALSE);
+  return(false);
 }
 
 /****************************************************************************
  *  Does a column of an alignment contain any ambiguity codes?
  ****************************************************************************/
-BOOLEAN_T alignment_site_ambiguous(int index, ALIGNMENT_T* alignment) {
+bool alignment_site_ambiguous(ALPH_T* alph, int index, ALIGNMENT_T* alignment) {
 
-  int i = 0;
+  int i;
   for (i = 0; i < alignment->num_sequences; i++) {
     char c = get_seq_char(index, alignment->sequences[i]);
-    if (is_ambiguous(c)) {
-      return(TRUE);
-    }
+    if (!alph_is_concrete(alph, c)) return(true);
   }
-  return(FALSE);
+  return(false);
 }
 
 /****************************************************************************
@@ -995,8 +1022,8 @@ static ALIGNMENT_T* remove_allgap_sequences(ALIGNMENT_T* alignment)
  ****************************************************************************/
 ALIGNMENT_T* read_alignment_from_file
   (char *filename, 
-   BOOLEAN_T sort,
-   BOOLEAN_T remove_allgap_seqs,
+   bool sort,
+   bool remove_allgap_seqs,
    int* ref_seq_index
   )
 {
