@@ -58,7 +58,7 @@ ARRAY_T* resize_init_array
 
   if (array == NULL) {
     new_array = allocate_array(num_items);
-    init_array(default_value, array);
+    init_array(default_value, new_array);
   } else {
     mm_resize(array->items, num_items, ATYPE);
     for (i = array->num_items; i < num_items; i++) array->items[i] = default_value;
