@@ -6,7 +6,10 @@
  * COPYRIGHT: 2011, UQ
  ********************************************************************/
 
+#ifdef MAIN
 #define DEFINE_GLOBALS
+#endif
+
 #include <assert.h>
 #include <getopt.h>
 #include <stdio.h>
@@ -48,8 +51,6 @@ const char* SITES_FILENAME = "site_counts.txt"; // Name of plain-text sites
 const char* TSV_FILENAME = "centrimo.tsv"; // Name of TSV centrimo output
 const char* HTML_FILENAME = "centrimo.html"; // Name of HTML centrimo output
 
-VERBOSE_T verbosity = NORMAL_VERBOSE;
-
 // Structure for tracking centrimo command line parameters.
 typedef struct options {
   ALPH_T* alphabet; // Alphabet
