@@ -16,7 +16,6 @@
 /* rounding stuff */
 #define RNDEPS 1e-12
 
-//FIXME:
 //bool debug = true;
 bool debug = false;
 
@@ -175,7 +174,7 @@ int get_best_nsites(
   n_nsites = j;
 
   // NOTE: If not Classic, I think this call is only needed to get the LLR.
-  // FIXME: Don't call align... if called from align...!
+  // TODO: Don't call align... if called from align...!
   if (objfun == Classic || objfun == NC) {
     /* get the probability that a site starting at position x_ij would
        NOT overlap a previously found motif; used in E_STEP.
@@ -428,7 +427,7 @@ static void set_pY(
 
       /* using inverse complement strand, too */
       if (pal) {				// use sum of Zi(-k)+Zi(k)
-        pY[jj] = INT_LOG(sw * MIN(1.0,(Zi(-k)+Zi(k))));	// FIXME??
+        pY[jj] = INT_LOG(sw * MIN(1.0,(Zi(-k)+Zi(k))));	// TODO??
       } else if (Zi(-k) > Zi(k)) {		// - strand
         pY[jj] = INT_LOG(sw * Zi(-k));
       } else {					// + strand
@@ -509,7 +508,7 @@ static void sort_maxima(
     SAMPLE *s = neg ? control->samples[x] : dataset->samples[x];
     int y = model->maxima[i].y;
     double score;
-//FIXME: Classic might be improved if it used erased LLR, too.
+//TODO: Classic might be improved if it used erased LLR, too.
 // TLB: I left this alone to keep Classic identical with v4.10.2
 // if sequence order shuffling is not done.
     if (dataset->objfun == Classic) {
@@ -574,7 +573,7 @@ static void sort_maxima(
     int y = model->maxima[i].y;
     SAMPLE *s = samples[x];				/* sequence */
     double sw = s->sw;					/* sequence weight */
-    // FIXME: We are assumming that priors are always symmetrical here.
+    // TODO: We are assumming that priors are always symmetrical here.
     double esw = sw * INT_DELOG(s->log_not_o[y]);     	// Pr(site not overlapped) * Pr(site)
     wN += esw;                                		/* total sequence wgt */
     model->maxima[i].wN = wN;				// weighted rank
@@ -802,7 +801,11 @@ static int classic_get_width_nsites_pvalue(
   //
   n_maxima = get_max(mtype, dataset, false, model->w, 0, model->maxima, invcomp, false);
 
-  if (n_maxima < min_nsites) return 0; 	// Exit if not enough sites.
+  // Return no sites if not enough sites.
+  if (n_maxima < min_nsites) {
+    model->nsites_dis = 0;
+    return(0);
+  }
 
   //
   // Sort the maxima.
@@ -904,7 +907,7 @@ static int classic_get_width_nsites_pvalue(
     set the best motif info in the model
   */
   model->w = best_w;				/* best width */
-  model->nsites_dis = MAX(MINSITES, best_nsites);	/* after discretization; not correct if samples held out */
+  model->nsites_dis = MIN(n_maxima, MAX(MINSITES, best_nsites));/* after discretization; not correct if samples held out */
   model->logpv = best_log_pv;			/* p-value */
   model->logev = log_ev;			/* E-value */
   model->llr = llr;				// LLR of best E-value
