@@ -422,9 +422,9 @@ int main (int argc, char *argv[])
   // Parse the command line.
   while (1) { 
     int c = 0;
-    char * option_value = NULL;
-    char * option_name = NULL;
-    const char * message = NULL;
+    char* option_value = NULL;
+    char* option_name = NULL;
+    const char* message = NULL;
 
     // Read the next option, and break if we're done.
     c = simple_getopt(&option_name, &option_value, &option_index);
@@ -432,7 +432,7 @@ int main (int argc, char *argv[])
       break;
     } else if (c < 0) {
     	simple_getopterror(&message);
-      die("Error processing command line options (%s)\n", message);
+      die("Error processing command line options: %s\n", message);
     }
 
     if (strcmp(option_name, "mintrans") == 0) {
