@@ -4,7 +4,7 @@
 *   Author: Timothy L. Bailey                                           *
 *                                                                       *
 *   Copyright                                                           *
-*   (2001-2014) Timothy L. Bailey                                       *
+*   (2001-2023) Timothy L. Bailey                                       *
 *                                                                       *
 *   All rights reserved.                                                * 
 *   Permission to use, copy, modify, and distribute any part of         *
@@ -61,7 +61,7 @@
   standard output.  It then prints the estimated p-values for all the
   scores, sorted in increasing order. 
 
-  The subroutine main() thus serves as an example of how to call ml_evd().
+  The subroutine main() thus serves as an example of how to call ml_evd()
   and fit_score_distribution();
 */
 /******************************************************************************/
@@ -89,6 +89,112 @@ char *OUTPUT_DIRNAME = "mcast_out";
 */
 /******************************************************************************/
 
+static int p_compare(
+  const void *v1,
+  const void *v2
+);
+
+static EVD_SET fitexp(
+  DISTR_T dtype,	// Type of distribution to compute
+  SCORE_SET score_set,  /* the set of scores and GC-contents */
+  int maxiter1,		/* maximum iterations for EM. */
+  int maxiter2,		/* maximum iterations for ML. */
+  int maxiter3,		/* maximum iterations for N-R. */
+  double eps1,		// Stopping criterion for EM.
+  double eps2,		// error tolerance for ML.
+  double eps3,		// error tolerance for N-R.
+  int min_scores,       // Minimum number of scores/range.
+  double max_ratio      // Maximum GC ratio between adj. ranges
+);
+
+static EVD_SET ml_evd(
+  SCORE_SET score_set,  /* the set of scores and lengths */
+  double H,             /* initial estimate for H */
+  int maxiter1,         /* maximum iterations for ML */
+  int maxiter2,         /* maximum iterations for N-R */
+  double eps1,          /* error tolerance for ML */
+  double eps2,          /* error tolerance for N-R */
+  int maxscores,        /* randomly select scores */
+  int size,             /* size of length ranges */
+  int min_scores,	// minimum number of scores per group
+  double lspan,         /* minimum length ratio/group */
+  double ethresh        /* outlier threshold */
+);
+
+static EVD ml_gcexp(
+  EVD evd,		// Contains parameters a, b, d to optimize.
+  SCORE_SET score_set,	// Scores.
+  int min_scores,	// minumum scores per bin
+  double mu1,		// GC-independent estimate of the mean
+  double *z,		// missing information
+  int maxiter1,		/* maximum iterations for ML */
+  int maxiter2,		/* maximum iterations for N-R */
+  double eps1,		/* error tolerance for ML */
+  double eps2		/* error tolerance for N-R */
+);
+
+static inline EVD nr_gcexp(
+  int star,		// which parameter to take the partial derivative with 
+			// respect to: 1 = a, 2 = b, 3 = d.
+  EVD evd,
+  int n,		// number of bins
+  double *w_mean_gc,	// binned weighted mean: gc_i
+  double *w_mean_s,	// binned weighted mean: s_i
+  double eps,		// stopping criterion
+  int maxiter		// max. allowed iterations
+);
+
+EVD ml_evd_nr(
+  EVD evd,               /* initial guess */
+  SCORE_SET score_set,   /* the set of scores and lens */
+  int maxiter1,          /* maximum iterations for ML */
+  int maxiter2,          /* maximum iterations for N-R */
+  double eps1,           /* error tolerance for ML */
+  double eps2,           /* error tolerance for N-R */
+  double ethresh,        /* outlier threshold */
+  int min_scores	// mininum number of scores
+);
+
+EVD get_est(
+  SCORE_SET score_set    /* set of scores */
+);
+
+EVD ml_lambda(
+  EVD evd,             /* current parameters of EVD */ 
+  SCORE_SET score_set, /* set of scores */
+  double eps,          /* stopping criterion */
+  int maxiter          /* max. allowed iterations */
+);
+
+EVD ml_H(
+  EVD evd,             /* current parameters of EVD */ 
+  SCORE_SET score_set, /* set of scores */
+  double eps,          /* stopping criterion */
+  int maxiter          /* max. allowed iterations */
+);
+
+DERIV dlambda(
+  EVD evd,             /* current parameters of EVD */ 
+  SCORE_SET score_set  /* set of scores */
+);
+
+DERIV dH(
+  EVD evd,             /* current parameters of EVD */ 
+  SCORE_SET score_set  /* set of scores */
+);
+
+#ifdef MAIN
+SCORE_SET read_scores(
+  DISTR_T dtype
+);
+#endif /* MAIN */
+
+/******************************************************************************/
+/*
+  subroutines
+*/
+/******************************************************************************/
+
 static void init_evd(EVD *evd) {
   evd->lambda = 0.0; // parameters
   evd->K = 0.0;
@@ -109,8 +215,11 @@ static void init_evd(EVD *evd) {
   evd->b = 0;		
   // Log likelihood (applies to both types of distributions).
   evd->L = 0.0;
-}
+} // init_evd
 
+//
+// Used when reading in a set of scores.
+//
 static void init_score_set(SCORE_SET *score_set) {
   score_set->n = 0;         // number of scores in set
   score_set->max_scores_saved = 0; // Array size for reservoir sampling.
@@ -173,122 +282,6 @@ void print_score_set(DISTR_T dtype, SCORE_SET score_set, EVD evd) {
 }
 #endif
 
-static int p_compare(
-  const void *v1,
-  const void *v2
-);
-
-static EVD_SET fitexp(
-  DISTR_T dtype,	// Type of distribution to compute
-  SCORE_SET score_set,  /* the set of scores and GC-contents */
-  int maxiter1,		/* maximum iterations for EM. */
-  int maxiter2,		/* maximum iterations for ML. */
-  int maxiter3,		/* maximum iterations for N-R. */
-  double eps1,		// Stopping criterion for EM.
-  double eps2,		// error tolerance for ML.
-  double eps3,		// error tolerance for N-R.
-  int min_scores,       // Minimum number of scores/range.
-  double max_ratio      // Maximum GC ratio between adj. ranges
-);
-
-static EVD_SET ml_evd(
-  SCORE_SET score_set,  /* the set of scores and lens */
-  double H,             /* initial estimate for H */
-  int maxiter1,         /* maximum iterations for ML */
-  int maxiter2,         /* maximum iterations for N-R */
-  double eps1,          /* error tolerance for ML */
-  double eps2,          /* error tolerance for N-R */
-  int maxscores,        /* randomly select scores */
-  int size,             /* size of length ranges */
-  int min_scores,	// minimum number of scores per group
-  double lspan,         /* minimum length ratio/group */
-  double ethresh        /* outlier threshold */
-);
-
-static EVD ml_gcexp(
-  EVD evd,		// Contains parameters a, b, d to optimize.
-  SCORE_SET score_set,	// Scores.
-  int min_scores,	// minumum scores per bin
-  double mu1,		// GC-independent estimate of the mean
-  double *z,		// missing information
-  int maxiter1,		/* maximum iterations for ML */
-  int maxiter2,		/* maximum iterations for N-R */
-  double eps1,		/* error tolerance for ML */
-  double eps2		/* error tolerance for N-R */
-);
-
-static inline EVD nr_gcexp(
-  int star,		// which parameter to take the partial derivative with 
-			// respect to: 1 = a, 2 = b, 3 = d.
-  EVD evd,
-  int n,		// number of bins
-  double *w_mean_gc,	// binned weighted mean: gc_i
-  double *w_mean_s,	// binned weighted mean: s_i
-  double eps,		// stopping criterion
-  int maxiter		// max. allowed iterations
-);
-
-static inline DERIV gcexp_dE_dstar (
-  int star,		// which parameter to take the partial derivative with respect to:
-  double a,		// star == 1
-  double b,		// star == 2
-  double d,		// star == 3
-  int n,		// number of bins
-  double *w_mean_gc,	// binned weighted mean
-  double *w_mean_s 	// binned weighted mean
-);
-
-EVD ml_evd_nr(
-  EVD evd,               /* initial guess */
-  SCORE_SET score_set,   /* the set of scores and lens */
-  int maxiter1,          /* maximum iterations for ML */
-  int maxiter2,          /* maximum iterations for N-R */
-  double eps1,           /* error tolerance for ML */
-  double eps2,           /* error tolerance for N-R */
-  double ethresh,        /* outlier threshold */
-  int min_scores	// mininum number of scores
-);
-
-EVD get_est (
-  SCORE_SET score_set    /* set of scores */
-);
-
-EVD ml_lambda(
-  EVD evd,             /* current parameters of EVD */ 
-  SCORE_SET score_set, /* set of scores */
-  double eps,          /* stopping criterion */
-  int maxiter          /* max. allowed iterations */
-);
-
-EVD ml_H(
-  EVD evd,             /* current parameters of EVD */ 
-  SCORE_SET score_set, /* set of scores */
-  double eps,          /* stopping criterion */
-  int maxiter          /* max. allowed iterations */
-);
-
-DERIV dlambda(
-  EVD evd,             /* current parameters of EVD */ 
-  SCORE_SET score_set  /* set of scores */
-);
-
-DERIV dH(
-  EVD evd,             /* current parameters of EVD */ 
-  SCORE_SET score_set  /* set of scores */
-);
-
-#ifdef MAIN
-SCORE_SET read_scores (
-  DISTR_T dtype
-);
-#endif /* MAIN */
-
-/******************************************************************************/
-/*
-  subroutines
-*/
-/******************************************************************************/
-
 /**********************************************************************/
 /*
   set_up_score_set
@@ -298,11 +291,11 @@ SCORE_SET read_scores (
 */
 /**********************************************************************/
 SCORE_SET *set_up_score_set(
-  PROB_T p_threshold,       // P-value threshold for motif hits.
-  PROB_T dp_threshold,      // Score threshold for DP.
-  int    max_gap,           // Maximum gap length to allow in matches.
-  bool negatives_only, // Negatives only in score set?
-  MHMM_T* the_hmm           // The HMM itself.
+  PROB_T p_threshold,		// P-value threshold for motif hits.
+  PROB_T dp_threshold,		// Score threshold for DP.
+  int    max_gap,		// Maximum gap length to allow in matches.
+  bool negatives_only,		// Negatives only in score set?
+  MHMM_T* the_hmm		// The HMM itself.
 )
 {
   int j;
@@ -498,7 +491,7 @@ static int p_compare(
  
   Compare two score gc's in ascending order.  Return <0 >0
   if the second is >, < the first.  If they are equal,
-  resolves ties by returning <0 if the second has smaller address.
+  resolves ties by returning <0 if the second has smaller serial number.
 */
 /**********************************************************************/
 static int gc_compare(
@@ -509,7 +502,7 @@ static int gc_compare(
   const SCORE *s1 = (const SCORE *) v1;
   const SCORE *s2 = (const SCORE *) v2;
   double diff = s2->gc - s1->gc;
-  if (diff == 0) diff = (double) (s1 - s2);
+  if (diff == 0) diff = s1->serial_no - s2->serial_no;
   return ((diff > 0) ? -1 : ( (diff < 0) ? 1 : 0) );
 } /* gc_compare */
 
@@ -531,7 +524,7 @@ static int gc_compare(
 /******************************************************************************/
 static EVD em_exp(
   SCORE_SET score_set,	/* the set of scores and GC contents */
-  DISTR_T dtype,	// type of distribution
+  DISTR_T dtype,	// type of distribution; D_EXP or D_GCEXP
   int maxiter1,		/* maximum iterations for EM. */
   int maxiter2,		/* maximum iterations for ML. */
   int maxiter3,		/* maximum iterations for N-R. */
@@ -566,7 +559,7 @@ static EVD em_exp(
   evd.max_t = max_gc;
 
   // Create missing information array.
-  mm_resize(z, n, double);
+  mm_resize(z, n+1, double);
 
   // Get sum of scores
   x_max = x_sum = 0;
@@ -601,13 +594,11 @@ static EVD em_exp(
     (n - n_big - 1) / (double) n;	// Fraction less than e_max.
 
   if (dtype == D_EXP) {			// Initilize EXP parameters.
-    mu1 = x_mean;			// Mean of (valid) data.
   } else {				// Intitialize GCEXP parameters.
-
     evd.a = x_mean;			// No GC dependence yet.
     evd.b = 0;
-    mu1 = x_mean;			// Mean of (valid) data.
   }
+  mu1 = x_mean;				// Mean of (valid) data.
   mu2 = e_max;				// Expected largest score.
   sigma2 = fabs(x_max-e_max)/2;		// Put largest out 2sd.
   sx = x_mean * n;			// Invariant over iterations.
@@ -661,10 +652,12 @@ static EVD em_exp(
     // M-step for exponential component.
     if (dtype == D_EXP) {			// binned-GC exponential
       mu1 = sxz/sz;	// Exponential mean.
-    } else {					// GC-dependent mean exponential
+    } else if (dtype == D_GCEXP) {		// GC-dependent mean exponential
       mu1 = sxz/sz; 	// Exponential mean.
       evd = ml_gcexp(evd, score_set, min_scores, mu1, z, maxiter2, maxiter3, eps1, eps3);
-      //mu1 = x_mean;	// Never changes; not really used.
+    } else {
+      fprintf(stderr, "ERROR: em_exp() called with wrong type of distribution.\n");
+      exit(1);
     }
 
     // M-step for Gaussian component
@@ -731,7 +724,7 @@ static EVD em_exp(
   scores from sequence regions with GC-contents in distinct ranges.
   The GC-content ranges are determined by:
     min_scores  -- minimum number of scores per range
-    max_ratio  -- maximum ratio of GC thresholds between ranges
+    max_ratio   -- maximum ratio of GC thresholds between ranges
            (use 1 range if max_ratio >= 100)
   Ranges are set up so that each range has at least min_scores in it.
   A new range i+1 is created when range i contains at least min_scores
@@ -755,7 +748,7 @@ static EVD em_exp(
 */
 /******************************************************************************/
 static EVD_SET fitexp(
-  DISTR_T dtype,	// Type of distribution to compute
+  DISTR_T dtype,	// Type of distribution to compute; D_EXP or D_GCEXP
   SCORE_SET score_set,  /* the set of scores and GC-contents */
   int maxiter1,		/* maximum iterations for EM. */
   int maxiter2,		/* maximum iterations for ML. */
@@ -797,8 +790,7 @@ static EVD_SET fitexp(
   score_set.scores = NULL;
   for (i=valid_n=0; i<n; i++, scores++) {
     // Span is too long if no room for extending match on each end.
-    if ( (scores->t - scores->span) < 2*score_set.max_gap) 
-      continue;        // Sequence too short; skip.
+    if ((scores->t - scores->span) < 2*score_set.max_gap) continue;	// Sequence too short; skip.
     if (valid_n % BLKSIZ == 0) {  /* make space in scores */
       mm_resize(score_set.scores, valid_n + BLKSIZ, SCORE);
     }
@@ -808,7 +800,7 @@ static EVD_SET fitexp(
 
   // Bail if not enough scores to do estimation.
   if (valid_n == 0 || valid_n < min_scores) {
-    sprintf( evd_set.msg, "Too few matches to estimate E-values." );
+    sprintf(evd_set.msg, "Too few matches (%d < %d) to estimate p-values.", valid_n, min_scores);
     fprintf(stderr, "%s\n", evd_set.msg);
     myfree(evds);
     myfree(score_set.scores);
@@ -824,6 +816,7 @@ static EVD_SET fitexp(
   i_low = 0;				// Start of current range of GC-content.
   gc_mid = scores[0].gc/2;		// mid-point GC of previous range.
   gc_hi = 0;				// Previous range maximum GC-content.
+
   for (g=0; g<nranges; g++) {		// Range.
     int count;				// Number of scores in range.
     double gc_sum = 0;			// Sum of GC-contents.
@@ -837,7 +830,7 @@ static EVD_SET fitexp(
         (i == valid_n - 1) || (		// Always end if last score.
           count >= min_scores &&	// Need at least min_scores
           (ratio >= max_ratio) &&	// and ratio reached
-          i < n-min_scores)		// and at least min_scores are left.
+          i < valid_n-min_scores)	// and at least min_scores are left.
         ) {
         // Set up score_set to contain just this range.
         score_set.scores = scores+i_low;
@@ -846,6 +839,16 @@ static EVD_SET fitexp(
         double min_gc = gc_hi;    		// Low GC is previous range hi GC.
         double max_gc = scores[i].gc;  		// Hi GC.
         evd = em_exp(score_set, dtype, maxiter1, maxiter2, maxiter3, eps1, eps2, eps3, min_gc, max_gc, old_min_scores);
+        // Flag problem if EM resulted in a negative value of mu1 for some point.
+        if (dtype==D_GCEXP && evd.n == 0) {
+          (void) sprintf(
+             evd_set.msg,
+             "Unable to estimate exponential using linear regression."
+          );
+          fprintf(stderr, "%s\n", evd_set.msg);
+          myfree(scores);
+          return(evd_set);
+        }
         evd.g = g; 
         evd.n = count;
         evd.mid_t = gc_sum/count;	// Mean GC.
@@ -859,7 +862,7 @@ static EVD_SET fitexp(
         if (dtype==D_EXP && evd.mu1 > evd.mu2) {
           (void) sprintf(
              evd_set.msg,
-             "Unable to estimate E-values because they don't follow "
+             "Unable to estimate p-values because they don't follow "
              "an exponential distribution (mu1 %10.2f, mu2 %10.2f).",
               evd.mu1, evd.mu2
           );
@@ -934,9 +937,9 @@ static EVD_SET fitexp(
 static EVD ml_gcexp(
   EVD evd,		// Contains parameters a, b, d to optimize.
   SCORE_SET score_set,	// Scores.
-  int min_scores,	// minumum scores per bin
+  int min_scores,	// minumum scores per bin; if 0, use raw points
   double mu1,		// GC-independent estimate of the mean
-  double *z,		// missing information
+  double *z,		// missing information; must have size score_set.n+1
   int maxiter1,		/* maximum iterations for ML */
   int maxiter2,		/* maximum iterations for N-R */
   double eps1,		/* error tolerance for ML */
@@ -951,43 +954,76 @@ static EVD ml_gcexp(
   int n = score_set.n;
   SCORE *scores = score_set.scores;    // Scores.
   int nbins = (min_scores > 0) ? n/min_scores : 0;	// number of bins with at least min_scores points
+  double mse = 0;
 
-  if (nbins == 0) {	// Use weighted linear regression on raw points
+  if (nbins == 0) {		// Use weighted linear regression on raw points
     // Move data into single X and Y arrays.
-    double *gc = NULL; mm_resize(gc, n, double);
-    double *s = NULL; mm_resize(s, n, double);
+    double *gc = NULL; mm_resize(gc, n+1, double);
+    double *s = NULL; mm_resize(s, n+1, double);
+    double mean_s = 0;
+    double sum_z = 0;
     for (i=0; i<n; i++) {
       gc[i] = scores[i].gc;
       s[i] = scores[i].s;
+      mean_s += z[i] * s[i];
+      sum_z += z[i];
     }
+    mean_s = sum_z > 0 ? mean_s/sum_z : 1;
     // Perform weighted regression on raw points.
-    w_regress(n, gc, s, z, &(evd.b), &(evd.a));
+    mse = w_regress(n, gc, s, z, &(evd.b), &(evd.a));
     //
     // Make sure MU1 is positive and add a point at gc=0 or gc=1 if not.
     //
-    int edge_index = -1;
-    double edge_gc;
-    if (MU1(0, evd.a, evd.b) <= 0) {
-      edge_index = 0;
+    double edge_gc = -1;
+    double mu1_0 = MU1(0, evd.a, evd.b);
+    double mu1_1 = MU1(1, evd.a, evd.b);
+    if (mu1_0 <= 0) {
       edge_gc = 0;
-    } else if (MU1(1, evd.a, evd.b) <= 0) {
-      edge_index = n-1;
+    } else if (mu1_1 <= 0) {
       edge_gc = 1;
     }
-    if (edge_index != -1) {		// MU1 non-positive
-      double z0 = z[edge_index];	// Save edge point weight
-      s[edge_index] += 1;		// Make edge score >= 1.
-      while (MU1(edge_gc, evd.a, evd.b) <= 0) {
+#ifdef DEBUG_FIT
+    fprintf(stderr, "weighted regression on raw points: n %d mse %g mu1(0) %g mu1(1) %g a=%g ; b=%g edge_gc %g\n",
+      n, mse, mu1_0, mu1_1, evd.a, evd.b, edge_gc);
+#endif
+    if (edge_gc != -1) {		// MU1 non-positive
+      n = n+1;				// Add extra point at end of list.
+      gc[n-1] = edge_gc;
+      s[n-1] = mean_s > 0 ? mean_s : 1;
+      //
+      // Binary search for fit that has mu1 > 0 at edge value of gc.
+      //
+      double mu1 = 0;
+      z[n-1] = 0.5;
+      double z_lo = 0;
+      double z_hi = 1;
+      bool z_upper_bound_known = false;
+      double a, b;
+      while (z_hi-z_lo > 0.01 && mse > 0) {
+        mse = w_regress(n, gc, s, z, &(evd.b), &(evd.a));	// Rerun weighted regression
+        mu1 = MU1(edge_gc, evd.a, evd.b);
         if (verbosity > NORMAL_VERBOSE) {
-          fprintf(stderr, "edge_index %d edge_gc %g gc %g score %g z %g a %g b %g\n", edge_index, edge_gc, gc[edge_index], s[edge_index], z[edge_index], evd.a, evd.b);
+          fprintf(stderr, "edge_gc %f z %g z_lo %g z_hi %g a %g b %g mu1(%g) %g\n", edge_gc, z[n-1], z_lo, z_hi, evd.a, evd.b, edge_gc, mu1);
+        }
+        if (mu1 <= 0) {
+          z_lo = z[n-1];					// New lower bound.
+          if (! z_upper_bound_known) {
+            z[n-1] = z_hi *= 2;					// Double upper bound.
+          } else {
+            z[n-1] += (z_hi-z_lo)/2;				// Test midpoint.
+          }
+        } else {
+          z_upper_bound_known = true;				// Have found an upper bound on z.
+          z_hi = z[n-1];					// New upper bound.
+          z[n-1] -= (z_hi-z_lo)/2;				// Test midpoint.
+          a = evd.a;						// Save last values with positive mu1.
+          b = evd.b;						// Save last values with positive mu1.
         }
-        z[edge_index] += 1;		// Upweight edge point.
-        gc[edge_index] += (edge_gc - gc[edge_index])/2;	// Move closer to edge_gc
-        w_regress(n, gc, s, z, &(evd.b), &(evd.a));	// Rerun weighted regression
       }
-      // Restore orginal edge point weight
-      z[edge_index] = z0;
+      evd.a = a;
+      evd.b = b;
     }
+
     myfree(gc);
     myfree(s);
   } else if (nbins == 1) {	// Use single mean ML estimate
@@ -1106,7 +1142,7 @@ static EVD_SET ml_evd(
   if (n < min_scores) {
     (void) sprintf(
       evd_set.msg,
-      "Too few (%d < %d) scores to estimate E-values.", 
+      "Too few (%d < %d) scores to estimate p-values.", 
       n, min_scores
     );
     fprintf(stderr, "%s\n", evd_set.msg);
@@ -1115,19 +1151,19 @@ static EVD_SET ml_evd(
     return(evd_set);
   }
 
-  /* subsample the scores if the size of the set exceeds the maximum
-     number of scores allowed
+  /* Subsample the scores if the size of the set exceeds the maximum
+     number of scores allowed.
   */
-  if (old_n > maxscores && maxscores > 0) {  /* too many scores */
-    double frac = 1.0-(maxscores+0.0)/n;  /* fraction to sample */
-    score_set.n = 0;        /* empty list of scores */
-    score_set.scores = NULL;      /* initialize score list */
-    srand_mt(0);          /* init rn generator */
-    for (i=0; i<n; i++) {       /* subsample scores */
-      if (drand_mt() >= frac) {      /* use this score */
-  if (score_set.n % BLKSIZ == 0) { 
-    mm_resize(score_set.scores, score_set.n + BLKSIZ, SCORE);
-  }
+  if (old_n > maxscores && maxscores > 0) {	/* too many scores */
+    double frac = 1.0-(maxscores+0.0)/n;	/* fraction to sample */
+    score_set.n = 0;				/* empty list of scores */
+    score_set.scores = NULL;			/* initialize score list */
+    srand_mt(0);				/* init rn generator */
+    for (i=0; i<n; i++) {			/* subsample scores */
+      if (drand_mt() >= frac) {			/* use this score */
+	if (score_set.n % BLKSIZ == 0) { 
+	  mm_resize(score_set.scores, score_set.n + BLKSIZ, SCORE);
+	}
         score_set.scores[score_set.n++] = scores[i];
       }
     } /* subsample scores */
@@ -1142,7 +1178,7 @@ static EVD_SET ml_evd(
   evd_init = get_est(score_set);
   evd_init.H = H <= MAXH ? H : MAXH;    /* limit size of H */
   if (evd_init.lambda == 0) {
-    (void) sprintf(evd_set.msg, "Too few scores to estimate E-values.");
+    (void) sprintf(evd_set.msg, "Too few scores to estimate p-values.");
     fprintf(stderr, "%s\n", evd_set.msg);
     myfree(t_hist);
     myfree(evds);
@@ -1152,11 +1188,11 @@ static EVD_SET ml_evd(
   /* 
     get length histogram of non-outlier scores 
   */
-  for (i=0; i<=max_t; i++) t_hist[i] = 0;  /* 0 counts of each length */
+  for (i=0; i<=max_t; i++) t_hist[i] = 0;	/* 0 counts of each length */
   for (i=total=0; i<n; i++) {
-    if (scores[i].pv != 1) {      /* score */
-      t_hist[scores[i].t]++;      /* # of non-outliers */
-      total++;          /* total non-outliers */
+    if (scores[i].pv != 1) {      		/* score */
+      t_hist[scores[i].t]++;			/* # of non-outliers */
+      total++;					/* total non-outliers */
     }
   } /* score */
 
@@ -1235,7 +1271,7 @@ static EVD_SET ml_evd(
 
     /* exit if error occured */
     if (evd.lambda == 0) {
-      (void) sprintf(evd_set.msg, "Too few scores to estimate E-values.");
+      (void) sprintf(evd_set.msg, "Too few scores to estimate p-values.");
       fprintf(stderr, "%s\n", evd_set.msg);
       return(evd_set);
     }
@@ -1560,7 +1596,7 @@ EVD ml_evd_nr(
     old_n = evd.n;        /* old number of non-outliers */
     evd.n = cur_score_set.n;      /* number of non-outliers */
     if (cur_score_set.n < min_scores) {
-      fprintf(stderr, "Too few scores to estimate E-values.\n");
+      fprintf(stderr, "Too few scores to estimate p-values.\n");
       evd.lambda = 0;        // Signal error.
       return(evd);
     }
@@ -1783,15 +1819,14 @@ DERIV dH(
 /*
   get_n
 
-  Get the expected number of scores for use in converting
-  p-values to E-values.
+  Get the expected number of scores for use in converting p-values to E-values.
 
   Returns n, the number of non-outliers.
 */
 /******************************************************************************/
 int get_n(
-  SCORE_SET score_set,      /* the set of scores and lens/gcs */
-  EVD_SET evd_set       // Distribution.
+  SCORE_SET score_set,		/* the set of scores and lens/gcs */
+  EVD_SET evd_set		// Distribution.
 ) 
 {
   int i;
@@ -1831,11 +1866,60 @@ int get_n(
   return(n-n_out);
 } // get_n
 
+/***********************************************
+ * Calculate the score distribution.
+ ***********************************************/
+EVD_SET calc_distr(
+  SCORE_SET score_set,		// Set of scores and lengths or GC contents.
+  DISTR_T dtype,		// Type of score distribution.
+  bool match_evalues		// Use match E-values?
+)
+{
+  double H = 0;			// don't use H
+  int maxiter1 = 10;		// max. iterations in L-H loop
+  int maxiter2 = 20;		// max. iterations in N-R loop
+  int size = 1000;		// size of length groups
+  double lspan = 1.5;		// min. length ratio/group
+  int min_scores = 0;
+
+  switch(dtype) {
+    case D_EVD:
+      lspan = 1.5;
+      min_scores = MIN_SCORES;
+      break;
+    case D_EXP:
+      lspan = 1.01;
+      min_scores = 1000;	// Use 1000 for more accuracy.
+      break;
+    case D_GCEXP:
+      lspan = 101;
+      min_scores = 0;		// use raw scores
+      break;
+    default:
+      die("Unknown distribution type.");
+      break;
+  }
+
+  //
+  // If using match E-values instead of score E-values, set all
+  // the sequence lengths to total_length/# matches.
+  //
+  if (dtype==D_EVD && match_evalues) {
+    int i;
+    int length = score_set.n ? score_set.total_length/score_set.n : score_set.total_length;
+    for (i=0; i<score_set.n; i++) score_set.scores[i].t = length;
+    score_set.max_t = length;
+  }
+
+  // Fit the distribution to the scores and lengths.
+  return(fit_score_distribution(dtype, score_set, H, maxiter1, maxiter2, EPS1, EPS2, size, min_scores, lspan));
+} // calc_distr
+
 /******************************************************************************/
 /*
   fit_score_distribution
 
-   Fit an exponential, gc-dependent-exponential or extreme value distribution
+   Fit an exponential, GC-dependent-exponential or extreme value distribution
    to a set of scores.
 
   See fitexp() and ml_evd() for meanings of parameters.
@@ -1881,7 +1965,7 @@ EVD_SET fit_score_distribution(
       (void) sprintf(
         evd_set.msg,
         "After removing outliers, too few (%d < %d) scores "
-        "remain to estimate E-values.",
+        "remain to estimate p-values.",
         evd_set.non_outliers, min_scores
       ); 
       fprintf(stderr, "%s\n", evd_set.msg);
@@ -2064,7 +2148,6 @@ int main(int argc, char **argv)
     return(1);
   }
   
-
   /* read in a set of scores and lengths */
   score_set = read_scores(dtype);	/* read scores */
   score_set.q = q;			/* set query length */
@@ -2078,7 +2161,6 @@ int main(int argc, char **argv)
     maxiter2,   /* maximum iterations for N-R */
     EPS1,       /* error tolerance for ML */
     EPS2,       /* error tolerance for N-R */
-    //maxscores,
     size,       /* size of length ranges */
     min_scores, // minimum number of scores per group
     lspan       /* minimum length ratio/group */
