@@ -1621,7 +1621,7 @@ void process_cmd_line(
     int c = 0;
     char* option_name = NULL;
     char* option_value = NULL;
-    const char * message = NULL;
+    const char* message = NULL;
 
     // Read the next option, and break if we're done.
     c = simple_getopt(&option_name, &option_value, &option_index);
@@ -1629,7 +1629,7 @@ void process_cmd_line(
       break;
     } else if (c < 0) {
       (void) simple_getopterror(&message);
-      die("Error processing command line options (%s)\n", message);
+      die("Error processing command line options: %s\n", message);
     }
     //output directory if not already
     if (strcmp(option_name, "o") == 0) {
