@@ -5,6 +5,11 @@
  * PROJECT: MHMM
  * DESCRIPTION: A simple program to test I/O of MHMMs.
  **************************************************************************/
+
+#ifdef MAIN
+#define DEFINE_GLOBALS
+#endif
+
 #include "utils.h"
 #include "mhmm-state.h"
 #include "read-mhmm.h" 
@@ -12,8 +17,6 @@
 #include "simple-getopt.h"
 #include <string.h>
 
-VERBOSE_T verbosity = INVALID_VERBOSE;
-
 /**************************************************************************
  * int main
  **************************************************************************/
@@ -54,11 +57,11 @@ int main (int argc, char *argv[])
       break;
     } else if (c < 0) {
     	simple_getopterror(&message);
-      die("Error processing command line options (%s)\n", message);
+      die("Error processing command line options: %s\n", message);
     }
 
     if (strcmp(option_name, "verbosity") == 0) {
-      verbosity = (VERBOSE_T)atoi(option_value);
+      verbosity = atoi(option_value);
     } 
   }
 
