@@ -15,6 +15,13 @@ typedef struct regexmotif {
 //  ARRAYLST_T* residues;
 }  REGEX_MOTIF_T;
 
+char *motifid2regexp(
+  char *motifid,
+  char crw,                     // central residue wildcar,
+  bool convert_x,               // convert 'x' to '.'
+  bool keep_mod                 // keep mod text or weight in regular expression
+);
+
 int get_counts_from_motifid(
   char *motifid,                                // contains RE plus "-", "." and numbers
   ARRAYLST_T* seqs,                             // sequences
