@@ -112,39 +112,12 @@ void momo_print_motifs(
       MOTIF_T* currmotif = currmotifinfo->motif;
       char* motifid = currmotif->id + 1;
 
-      char* motif_name = mm_malloc(strlen(motifid) + 2);
-      char *motif_regexp = NULL;
-      // Add a central 'X' in the motif name if doing single motif per mass.
-      for (k=m=0; k < strlen(motifid); k++) {
-        // Check for single motif per mass (may be "__" or "_number").
-        if ( (k == 0) && (motifid[0]-'0' >= 0 && motifid[0]-'0' <= 9) ) {
-          // Add missing central 'X' to name.
-          motif_name[m++] = 'X';
-          motif_name[m++] = motifid[0];
-	} else if (motifid[k] == '_' && (motifid[k+1] == '_' || (motifid[k+1]-'0' >= 0 && motifid[k+1]-'0' <= 9))) { 
-          // Convert '_' to 'X' for (missing) central residue.
-          motif_name[m++] = '_';
-          motif_name[m++] = 'X';
-	} else {
-	  motif_name[m++] = motifid[k];
-	}
-      }
-      motif_name[m] = '\0';
+      char *motif_name = motifid2regexp(motifid, 'X', false, true);
       strncpy(currmotif->id+1, motif_name, MAX_MOTIF_ID_LENGTH);
 
+      char *motif_regexp = NULL;
       if (options.algorithm == Motifx || options.algorithm == Modl) {
-        motif_regexp = mm_malloc(strlen(motifid) + 2);
-        // Convert motif_name to PERL regular expression.
-	for (k=m=0; k < strlen(motif_name); k++) {
-	  if (motif_name[k] == 'x' || motif_name[k] == 'X') {
-	    motif_regexp[m++] = '.';
-	  } else if (motif_name[k]-'A' >= 0 && motif_name[k]-'Z' <= 0) {
-	    motif_regexp[m++] = motif_name[k];
-	  } else if (motif_name[k] == '[' || motif_name[k] == ']') {
-	    motif_regexp[m++] = motif_name[k];
-	  }
-	}
-	motif_regexp[m] = '\0';
+        motif_regexp = motifid2regexp(motifid, '.', true, false);
       } else {				// algorithm = simple
         // Create expanded motif name.
         int w = options.width;
