@@ -19,7 +19,7 @@
 
 extern void mpInit(
   int *argc,
-  char **argv[]
+  char ***argv
 );
 extern void mpFinalize();
 extern int mpNodes();
