@@ -63,6 +63,12 @@ void* retrieve_next_object
 ARRAY_T* get_object_list_scores
   (OBJECT_LIST_T* a_list);
 
+/*************************************************************************
+ *  Get the number of objects in an object list.
+ *************************************************************************/
+int get_object_list_length
+  (OBJECT_LIST_T* a_list);
+
 /*************************************************************************
  * Free dynamic memory used by a list of objects.
  *************************************************************************/
