@@ -19,7 +19,8 @@
  *****************************************************************************/
 DATA_BLOCK_READER_T *new_prior_reader_from_psp(
   bool parse_genomic_coord, 
-  const char *filename
+  const char *filename,
+  double default_prior
 );
 
 #endif
