@@ -9,11 +9,17 @@
  *
 */
 
+#ifdef MAIN
+#define DEFINE_GLOBALS
+#endif
+
 #include <assert.h>
 #include <stdio.h>
 #include <string.h>
 #include "ranksum_test.h"
 #include "utils.h"
+#include "log_erfc.h"
+#include "io.h"
 
 #define min(a,b)      (a<b)?a:b
 #define DATA_INCREMENT  1000
@@ -26,7 +32,7 @@
   this struct contains the results of a ranksum calculation
 **********************************************************************/
 struct ranksum_result{
-	double p_left, p_right, p_twotailed; // p_values
+	double log_p_left, log_p_right, log_p_twotailed; // log p_values
 	double u;			// u values
 };
 
@@ -39,7 +45,7 @@ struct ranksum_result{
 struct ranksum_datapoint{
 	double score;   // measurement
 	double rank;	// the rank
-	BOOLEAN_T group;// group
+	bool group;// group
 };
 
 /**********************************************************************
@@ -81,7 +87,7 @@ RSD_T *allocate_and_init_ranksum_data(unsigned int size);
 
   Constructor for the ranksum data structure
 **********************************************************************/
-RSDP_T *allocate_ranksum_datapoint(double score, BOOLEAN_T group);
+RSDP_T *allocate_ranksum_datapoint(double score, bool group);
 
 /**********************************************************************
   allocate_ranksum_datapoint()
@@ -112,13 +118,13 @@ void free_rsd(RSD_T *dataset);
   Constructor for the ranksum result data structure
 **********************************************************************/
 RSR_T *allocate_ranksum_result(){
-	// Allocate memory and initialize fields
-	RSR_T *data = mm_malloc(sizeof(RSR_T));
-	data->u = 0.0;
-	data->p_left = 0.0;
-	data->p_right = 0.0;
-	data->p_twotailed = 0.0;
-	return data;
+  // Allocate memory and initialize fields
+  RSR_T *data = mm_malloc(sizeof(RSR_T));
+  data->u = 0.0;
+  data->log_p_left = -BIG;
+  data->log_p_right = -BIG;
+  data->log_p_twotailed = -BIG;
+  return data;
 };
 
 /**********************************************************************
@@ -128,7 +134,7 @@ RSR_T *allocate_ranksum_result(){
 **********************************************************************/
 RSDP_T *allocate_ranksum_datapoint(
 		double score,
-		BOOLEAN_T group
+		bool group
 ){
 	// Allocate memory and initialize fields
 	RSDP_T *datapoint = mm_malloc(sizeof(RSDP_T));
@@ -144,13 +150,13 @@ RSDP_T *allocate_ranksum_datapoint(
   Constructor for the ranksum data structure
 **********************************************************************/
 RSD_T *allocate_ranksum_data(){
-	// Allocate memory and initialize fields
-	RSD_T *data = mm_malloc(sizeof(RSD_T));
+  // Allocate memory and initialize fields
+  RSD_T *data = mm_malloc(sizeof(RSD_T));
 
-	data->size = 0;
-	data->allocated_size = 0;
-	data->data_points = NULL;
-	return data;
+  data->size = 0;
+  data->allocated_size = 0;
+  data->data_points = NULL;
+  return data;
 }
 
 /**********************************************************************
@@ -160,13 +166,13 @@ RSD_T *allocate_ranksum_data(){
   up to size datapoints
 **********************************************************************/
 RSD_T *allocate_and_init_ranksum_data(unsigned int size){
-	// Allocate memory and initialize fields
-	RSD_T *data = mm_malloc(sizeof(RSD_T));
+  // Allocate memory and initialize fields
+  RSD_T *data = mm_malloc(sizeof(RSD_T));
 
-	data->size = 0;
-	data->allocated_size = size;
-	data->data_points = (RSDP_T **) mm_malloc(data->allocated_size * sizeof(RSDP_T *));
-	return data;
+  data->size = 0;
+  data->allocated_size = size;
+  data->data_points = (RSDP_T **) mm_malloc(data->allocated_size * sizeof(RSDP_T *));
+  return data;
 }
 
 /**********************************************************************
@@ -175,42 +181,43 @@ RSD_T *allocate_and_init_ranksum_data(unsigned int size){
   Computes the ranksum test for a given statistic.
 **********************************************************************/
 RSR_T *ranksum_from_stats(
-   int n,			/* number of samples */
-   int na,			/* number of positives */
+   int n,		/* number of samples */
+   int na,		/* number of positives */
    double ta_obs	/* sum of ranks of positives */
 ){
-	RSR_T *mww = allocate_ranksum_result();
+  RSR_T *mww = allocate_ranksum_result();
 
-	int nb = n-na;
+  int nb = n-na;
 
-	double tab=n*(n+1)*0.5; 				// sum of n ranks in groups A and B combined
-	double tb_obs = tab - ta_obs;
+  double tab=n*(n+1)*0.5; 				// sum of n ranks in groups A and B combined
+  double tb_obs = tab - ta_obs;
 
-	double sd=sqrt((na*nb*(n+1.0))/12.0); 	// the standard deviation is the same in both sets
-	double ta_null=na*(n+1.0)/2.0;			// the sum of the "null" case
-	double tb_null=nb*(n+1.0)/2.0;			// the sum of the "null" case
-	double ta_max=na*nb+(na*(na+1.0))/2.0;	// the max sum set A can take
-	double tb_max=na*nb+(nb*(nb+1.0))/2.0;	// the max sum set B can take
-	double ua=ta_max-ta_obs;				// the "U" value for A
-	double ua_null=ta_max-ta_null;			// the U value for the null case
-	double da=ta_obs>ta_null?-0.5:+0.5;		// a "continuity correction" for A
-	double db=tb_obs>tb_null?-0.5:+0.5;		// a "continuity correction" for B
-	double za=((ta_obs-ta_null)+da)/sd;		// the z value for A which is the mirror of ...
-	double zb=((tb_obs-tb_null)+db)/sd;		// the z value for B (we only need one)
-	double pa=0.5*(erfc(za/sqrt2));			// figure out the area of the normal distribution
-	double pb=0.5*(erfc(zb/sqrt2));			// figure out the area of the normal distribution
+  double sd=sqrt((na*nb*(n+1.0))/12.0); 		// the standard deviation is the same in both sets
+  double ta_null=na*(n+1.0)/2.0;			// the sum of the "null" case
+  double tb_null=nb*(n+1.0)/2.0;			// the sum of the "null" case
+  double ta_max=na*nb+(na*(na+1.0))/2.0;		// the max sum set A can take
+  double tb_max=na*nb+(nb*(nb+1.0))/2.0;		// the max sum set B can take
+  double ua=ta_max-ta_obs;				// the "U" value for A
+  double ua_null=ta_max-ta_null;			// the U value for the null case
+  double da=ta_obs>ta_null?-0.5:+0.5;			// a "continuity correction" for A
+  double db=tb_obs>tb_null?-0.5:+0.5;			// a "continuity correction" for B
+  double za=((ta_obs-ta_null)+da)/sd;			// the z value for A which is the mirror of ...
+  double zb=((tb_obs-tb_null)+db)/sd;			// the z value for B (we only need one)
+  //double pa=0.5*(erfc(za/sqrt2));			// figure out the area of the normal distribution
+  //double pb=0.5*(erfc(zb/sqrt2));			// figure out the area of the normal distribution
+  double log_pa = log(0.5) + log_erfc(za/sqrt2);	// figure out the area of the normal distribution
+  double log_pb = log(0.5) + log_erfc(zb/sqrt2);	// figure out the area of the normal distribution
 
-	if (pa < pb){							// make sure we get the accuracy on
-		mww->p_right = pa;					// the smaller of the p-values
-		mww->p_left = 1.0-pa;				// due to numerical issues
-		mww->p_twotailed = 2.0 * pa;
-	} else {
-		mww->p_left = pb;
-		mww->p_right = 1.0-pb;
-		mww->p_twotailed = 2.0 * pb;
-	}
-	mww->u = ua;
-	return mww;
+  if (log_pa < log_pb){					// make sure we get the accuracy on
+    mww->log_p_right = log_pa;				// the smaller of the log p-values
+    mww->log_p_left = log(1.0 - exp(log_pa));		// due to numerical issues
+  } else {
+    mww->log_p_left = log_pb;
+    mww->log_p_right = log(1.0 - exp(log_pb));
+  }
+  mww->log_p_twotailed = mww->log_p_right < mww->log_p_left ? log(2)+mww->log_p_right : log(2)+mww->log_p_left;
+  mww->u = ua;
+  return mww;
 }
 
 /*************************************************************************
@@ -218,7 +225,7 @@ RSR_T *ranksum_from_stats(
  *
  * Compares two ranksum_data structures
  *************************************************************************/
-inline int ranksum_data_cmp(const void *v1, const void *v2)
+int ranksum_data_cmp(const void *v1, const void *v2)
 {
   assert(v1 != NULL);
   assert(v2 != NULL);
@@ -230,7 +237,8 @@ inline int ranksum_data_cmp(const void *v1, const void *v2)
   } else if (diff > 0.0){
   	  return -1;
   } else {
-  	  return 0;
+  	  //return 0;
+  	  return(s1 < s2 ? 1 : 0);	// resolve ties
   }
 }
 
@@ -259,6 +267,50 @@ void add_ranksum_datapoint(
   data->size++;
 }
 
+/**********************************************************************
+  get_ranksum_dataset2()
+
+  Creates a ranksum input data set that can be (re-)used with the
+  function run_ranksum_test() or run_ranksum_test_on_ordered_dataset()
+
+  Copies the list of samples and groups to the new data structure.
+  Calculates the (adjusted) rank of each score.
+
+  NOTE: The samples are expected to be sorted already. The arrays
+        of samples and groups have to be of the specified size and each
+        entry in group corresponds to the sample at the same index.
+**********************************************************************/
+RSD_T *get_ranksum_dataset2(
+	void* samples,  	/* the sample set, allows NULL */
+  double (*get_sample)(const void*, int), /* sample accessor */
+	void* group,	/* set of the group each samples belongs to, allows NULL */
+  bool (*get_group)(const void*, int), /* group accessor */
+	int size 			/* the size of sample set  */
+){
+	assert(size > 0);
+	int i;
+	// Create data structure 
+	RSD_T *dataset = allocate_and_init_ranksum_data(size);
+	// Fill data structure
+	for (i = 0; i < size; ++i) {
+		RSDP_T* datapoint = allocate_ranksum_datapoint(get_sample(samples, i), get_group(group, i));
+		add_ranksum_datapoint(dataset, datapoint);
+	}
+	// adjust rankings for same value elements
+	adjust_ranking(dataset);
+	return (dataset);
+}
+
+/*
+ * Internal functions to get items from arrays of type double and boolean
+ */
+double double_array_get(const void *array, int index) {
+  return ((double*)array)[index];
+}
+bool boolean_array_get(const void *array, int index) {
+  return ((bool*)array)[index];
+}
+
 /**********************************************************************
   get_ranksum_dataset()
 
@@ -274,29 +326,13 @@ void add_ranksum_datapoint(
 **********************************************************************/
 RSD_T *get_ranksum_dataset(
 	double* samples,  	/* the sample set */
-	BOOLEAN_T* group,	/* set of the group each samples belongs to */
+	bool* group,	/* set of the group each samples belongs to */
 	int size 			/* the size of sample set  */
 ){
-	assert(samples != NULL);
-	assert(group != NULL);
-	assert(size > 0);
-
-	int i;
-
-	/* Create data structure */
-	RSD_T *dataset = allocate_and_init_ranksum_data(size);
-
-	/* Fill data structure */
-	for (i=0;i<size;++i){
-		RSDP_T* datapoint = allocate_ranksum_datapoint(samples[i], group[i]);
-		add_ranksum_datapoint(dataset, datapoint);
-	}
-	// adjust rankings for same value elements
-	adjust_ranking(dataset);
-
-	return (dataset);
+  assert(samples != NULL);
+  assert(group != NULL);
+	return get_ranksum_dataset2(samples, double_array_get, group, boolean_array_get, size);
 }
-
 /**********************************************************************
   ranksum_from_groups()
 
@@ -305,7 +341,7 @@ RSD_T *get_ranksum_dataset(
 **********************************************************************/
 RSR_T *ranksum_from_groups(
 		double* samples,  	/* the sample set */
-		BOOLEAN_T* group,	/* set of the group each samples belongs to */
+		bool* group,	/* set of the group each samples belongs to */
 		int size 			/* the size of sample set  */
 ){
 	assert(samples != NULL);
@@ -354,11 +390,11 @@ RSR_T *ranksum_from_sets(
 
 	/* Fill data structure */
 	for (i=0;i<size_a;++i){
-		RSDP_T* datapoint = allocate_ranksum_datapoint(sample_a[i], TRUE);
+		RSDP_T* datapoint = allocate_ranksum_datapoint(sample_a[i], true);
 		add_ranksum_datapoint(dataset, datapoint);
 	}
 	for (i=0;i<size_b;++i){
-		RSDP_T* datapoint = allocate_ranksum_datapoint(sample_b[i], FALSE);
+		RSDP_T* datapoint = allocate_ranksum_datapoint(sample_b[i], false);
 		add_ranksum_datapoint(dataset, datapoint);
 	}
 
@@ -395,13 +431,36 @@ RSR_T* run_ranksum_test(
   run_ranksum_test_on_ordered_dataset i.e. when the ranking (ordering)
   does not change between the ranksum tests but the groups (classes) do.
 **********************************************************************/
-void set_ranksum_groups(RSD_T *dataset, BOOLEAN_T* groups){
+void set_ranksum_groups(RSD_T *dataset, bool* groups){
 	int i;
 	for (i=0;i<dataset->size;++i){
 		dataset->data_points[i]->group = groups[i];
 	}
 }
 
+/**********************************************************************
+  set_ranksum_group()
+
+  Assigns the given position to the given group in the dataset structure. 
+  For obvious reasons the position must exist.
+
+  Most beneficial when used in conjuction with get_ranksum_dataset and
+  run_ranksum_test_on_ordered_dataset i.e. when the ranking (ordering)
+  does not change between the ranksum tests but the groups (classes) do.
+**********************************************************************/
+void set_ranksum_group(RSD_T *dataset, int position, bool group){
+  dataset->data_points[position]->group = group;
+}
+
+/********************************************************************
+  get_ranksum_rank()
+
+  Gets the rank of the position
+**********************************************************************/
+double get_ranksum_rank(RSD_T *dataset, int position) {
+  return dataset->data_points[position]->rank;
+}
+
 /**********************************************************************
   adjust_ranking()
 
@@ -455,40 +514,40 @@ RSR_T* run_ranksum_test_on_ordered_dataset(
 }
 
 /**********************************************************************
-  RSR_get_p_left()
+  RSR_get_log_p_left()
 
-  returns the p-value for the left side (lesser)
+  returns the log p-value for the left side (lesser)
 **********************************************************************/
-double RSR_get_p_left(RSR_T* ranksum_result){
-	return ranksum_result->p_left;
+double RSR_get_log_p_left(RSR_T* ranksum_result){
+	return ranksum_result->log_p_left;
 }
 
 /**********************************************************************
-  RSR_get_p_right()
+  RSR_get_log_p_right()
 
-  returns the p-value for the right side (greater)
+  returns the log p-value for the right side (greater)
 **********************************************************************/
-double RSR_get_p_right(RSR_T* ranksum_result){
-	return ranksum_result->p_right;
+double RSR_get_log_p_right(RSR_T* ranksum_result){
+	return ranksum_result->log_p_right;
 }
 
 /**********************************************************************
-  RSR_get_p_onetailed()
+  RSR_get_log_p_onetailed()
 
-  returns the p-value for the one tailed side (the smaller one of left
+  returns the log p-value for the one tailed side (the smaller one of left
   and right)
 **********************************************************************/
-double RSR_get_p_onetailed(RSR_T* ranksum_result){
-	return min(ranksum_result->p_right,ranksum_result->p_left);
+double RSR_get_log_p_onetailed(RSR_T* ranksum_result){
+	return min(ranksum_result->log_p_right,ranksum_result->log_p_left);
 }
 
 /**********************************************************************
-  RSR_get_p_twotailed()
+  RSR_get_log_p_twotailed()
 
-  returns the p-value for the twotailed side
+  returns the log p-value for the twotailed side
 **********************************************************************/
-double RSR_get_p_twotailed(RSR_T* ranksum_result){
-	return ranksum_result->p_twotailed;
+double RSR_get_log_p_twotailed(RSR_T* ranksum_result){
+	return ranksum_result->log_p_twotailed;
 }
 
 /**********************************************************************
@@ -508,9 +567,9 @@ double RSR_get_u(RSR_T*  ranksum_result){
   instead.
 **********************************************************************/
 void free_rsd(RSD_T *dataset){
-	myfree(dataset->data_points);
-	dataset->size = 0;
-	myfree(dataset);
+  myfree(dataset->data_points);
+  dataset->size = 0;
+  myfree(dataset);
 }
 
 /**********************************************************************
@@ -521,14 +580,14 @@ void free_rsd(RSD_T *dataset){
   instead.
 **********************************************************************/
 void destroy_rsd(RSD_T *dataset){
-	while(dataset->size > 0) {
-		RSDP_T *dp = dataset->data_points[--dataset->size] ;
-		myfree(dp);
-	}
-	myfree(dataset->data_points);
-	dataset->size = 0;
+  while(dataset->size > 0) {
+    RSDP_T *dp = dataset->data_points[--dataset->size] ;
+    myfree(dp);
+  }
+  myfree(dataset->data_points);
+  dataset->size = 0;
 
-	myfree(dataset);
+  myfree(dataset);
 }
 
 /**********************************************************************
@@ -537,7 +596,7 @@ void destroy_rsd(RSD_T *dataset){
   Frees the memory of a RSR_T* data structure
 **********************************************************************/
 void destroy_rsr(RSR_T *dataset){
-	myfree(dataset);
+  myfree(dataset);
 }
 
 /*****************************************************************************
@@ -546,7 +605,6 @@ void destroy_rsr(RSR_T *dataset){
 #ifdef MAIN
 
 #include "simple-getopt.h"
-VERBOSE_T verbosity = INVALID_VERBOSE;
 #define MAX_LINE 1000
 
 int main
@@ -572,7 +630,7 @@ int main
 
   // Define the usage message.
   char      usage[400] = "";
-  strcat(usage, "USAGE: ranksum_test [options] <n> <p> <r>\n");
+  strcat(usage, "Usage: ranksum_test [options] <n> <p> <r>\n");
   strcat(usage, "\n");
   strcat(usage, "   <n> number of samples \n");
   strcat(usage, "   <p> number of positives\n");
@@ -588,7 +646,7 @@ int main
   int option_index = 0;
   char* option_name = NULL;
   char* option_value = NULL;
-  const char *  message = NULL;
+  const char* message = NULL;
   char line[MAX_LINE];    // Buffer for reading.
   simple_setopt(argc, argv, num_options, options);
   while(1) {
@@ -598,7 +656,7 @@ int main
       break;
     } else if (c < 0) {
       simple_getopterror(&message);
-      die("Error processing command line options (%s)\n", message);
+      die("Error processing command line options: %s\n", message);
     }
 
     if (strcmp(option_name, "p-value") == 0) {
@@ -613,7 +671,7 @@ int main
 
   // Read the single required argument.
   if (option_index + 3 != argc) {
-    fprintf(stderr, usage);
+    fprintf(stderr, "%s", usage);
     exit(1);
   }
   int n = atoi(argv[option_index]);option_index++;
@@ -624,20 +682,23 @@ int main
 
   // Print to stdout.
   if (requested_p_value == P_RIGHT){
-	  printf("%g\n",RSR_get_p_right(mww));
+    print_log_value(stdout, RSR_get_log_p_right(mww), 2);
   } else if (requested_p_value == P_LEFT){
-	  printf("%g\n",RSR_get_p_left(mww));
+    print_log_value(stdout, RSR_get_log_p_left(mww), 2);
   } else if (requested_p_value == P_ONETAILED){
-	  printf("%g\n",RSR_get_p_onetailed(mww));
+    print_log_value(stdout, RSR_get_log_p_onetailed(mww), 2);
   } else if (requested_p_value == P_TWOTAILED){
-	  printf("%g\n",RSR_get_p_twotailed(mww));
+    print_log_value(stdout, RSR_get_log_p_twotailed(mww), 2);
   } else {
-	  printf("p-left\t%g\tp-right\t%g\tone-tailed\t%g\ttwo-tailed\t%g\tU-value\t%g\n",
-			  RSR_get_p_left(mww),
-			  RSR_get_p_right(mww),
-			  RSR_get_p_onetailed(mww),
-			  RSR_get_p_twotailed(mww),
-			  RSR_get_u(mww));
+    printf("p-left\t");
+    print_log_value(stdout, RSR_get_log_p_left(mww), 2);
+    printf("\tp-right\t");
+    print_log_value(stdout, RSR_get_log_p_right(mww), 2);
+    printf("\tp-one-tailed\t");
+    print_log_value(stdout, RSR_get_log_p_onetailed(mww), 2);
+    printf("\tp-two-tailed\t");
+    print_log_value(stdout, RSR_get_log_p_twotailed(mww), 2);
+    printf("\tU-value\t%g\n", RSR_get_u(mww));
   }
   return(0);
 }
