@@ -109,7 +109,7 @@ static void usage(SPAMO_OPTIONS_T *options, char *format, ...) {
 
   char *usage = 
     "Usage:\n" 
-    "    spamo [options] <sequences> <primary motif> <secondary motifs>+\n"
+    "    spamo [options] <sequences> <primary motif db> <secondary motif db>+\n"
     "Options:\n"
     "    -o <directory>   create the directory and write output files in it;\n"
     "                       not compatible with -oc; default: -oc spamo_out\n"
