@@ -39,6 +39,7 @@
 #include "motif-db.h"
 #include "xml-out.h"
 #include "projrel.h"	// for ARCHIVE_DATE
+#include "seq-reader-from-fasta.h"
 
 // Imported from st_construct.c
 Sint constructstree(Suffixtree *stree, SYMBOL *text, Uint textlen);
@@ -107,7 +108,7 @@ void mmcheckspaceleak(void);
 typedef enum {DE, CD, NO_OBJFUN} OBJFUN_T;
 
 // Saving matches type.
-typedef enum {NONE, ZOOPS, PASSING} SAVE_MATCHES_T;
+typedef enum {NONE, ZOOPS, PASSING, APPEND_PASSING} SAVE_MATCHES_T;
 
 // Site positional distribution alignment type.
 typedef enum {LEFT, CENTER, RIGHT} SEQ_ALIGN_T;
@@ -412,6 +413,7 @@ typedef struct {
   Uint pos;		// position of site in input sequence
   Uchar strand;		// '-': site is on negative (not input) strand
   BOOL is_positive;	// true if sequence is positive
+  BOOL is_holdout;	// true if sequence is in test_multiseq
   double score;		// score of site
   Uint pos_count;	// number of positive sequences with this site
   Uint neg_count;	// number of negative sequences with this site
@@ -471,6 +473,7 @@ typedef struct {
   // matching sequences
   Passing_seq *passing_seqs;			// sequences with a passing site
   int npassing;					// number of sequences with a passing site
+  int npos_passing;				// number of positive sequences with a passing site
   // other
   BOOL is_palindromic;				// model is palindromic
   double ed;					// palindrome edit distance
@@ -590,6 +593,7 @@ typedef struct options {
 //
 // Options for SEA only:
   FILE *sequences_tsv_output;		// name of TSV file for the passing sequences of each significant motif
+  FILE *sites_tsv_output;		// name of TSV file for the positive sites
   BOOL noseqs;				// don't output passing sequences if true
   ARRAYLST_T *motif_sources;            // filenames of the motif library
   ARRAYLST_T *include_patterns;         // Wildcard patterns of motif names to include
@@ -612,6 +616,7 @@ Multiseq *read_pos_neg_seqs(
   BOOL allow_ambigs,                    // don't convert ambiguous characters to SEPARATOR
   BOOL no_trim,                         // don't trim sequences to average length
   BOOL set_back,                        // set the background in the multiseq objects
+  BOOL reset_rand,                      // reset the random number generator
   ARRAY_T *background,                  // IN MEME-style background model
   Multiseq **test_multiseq_ptr          // OUT hold-out set
 ); 
@@ -649,7 +654,7 @@ void score_model_pssm(
   SAVE_MATCHES_T save_matches,          // ZOOPS: save best match (sorted) in every sequence (positive or negative) for nesting
                                         // PASSING: save all matches with score >= model->score_threshold for erasing
   BOOL is_ho,                           // data is the hold-out set
-  BOOL use_cache,                        // use the p-value cache
+  BOOL use_cache,                       // use the p-value cache
   BOOL pos_only                         // just score the positive sequences
 );
 
@@ -698,4 +703,14 @@ char *get_description_file(char *filename);
 // Imported from st_mapfile.c
 void mmcheckspaceleak(void);
 
+//
+// Print positive sites for each motif in TSV format.
+//
+void print_streme_positive_sites_tsv(
+  STREME_OPTIONS_T *options,    // STREME options
+  const char *sites_tsv_path,   // path to positive sites TSV output file
+  Model **models,               // the discovered motifs
+  int n_out_motifs              // the number of motifs to output
+);
+
 #endif
