@@ -10,15 +10,19 @@
 #ifndef SUBST_MATRIX_H
 #define SUBST_MATRIX_H
 
+#include "matrix.h"
+#include "alphabet.h"
+#include "array.h"
+
 MATRIX_T *get_subst_target_matrix(
   char *score_filename,		/* name of score file */
-  BOOLEAN_T dna,		/* DNA alphabet if true; default: protein */
+  ALPH_T* alph,                  /* alphabet */
   int dist,			/* PAM distance (ignored if score_filename != NULL) */
-  char *alpha,			/* standard alphabet for final matrix */
   ARRAY_T *back			/* background frequencies of standard alphabet */
 );
 
 ARRAY_T *get_pseudocount_freqs(
+   ALPH_T* alph,
    ARRAY_T *      f,            /* Foreground distribution. */
    ARRAY_T *      b,            /* Background distribution. */
    MATRIX_T *     target_freq   /* Target frequency matrix. */
