@@ -1,4 +1,3 @@
-
 // Custom error
 var AlphabetError = function(message, reasons) {
   "use strict";
@@ -400,6 +399,13 @@ AlphabetParser.prototype.parse_done = function() {
 
 AlphabetParser.prototype.process_blob = function (blob, handler) {
   "use strict";
+  // file name error
+  if (blob.name != null) {
+    var extension = blob.name.split('.').pop();
+    if (forbidden.has(extension)) {
+      this.handler.error(true, "File name extension is illegal: '" + extension + "'.");
+    }
+  }
   // file name length error
   if (handler.max_file_name_length != null && blob.name != null && blob.name.length > handler.max_file_name_length) {
     this.handler.error(true, "The alphabet file name is too long: " + blob.name.length + " > " + handler.max_file_name_length + ".");
