@@ -537,6 +537,13 @@ BgHandler.prototype.summary = function () {
     if (is_error) error = true;
     else warning = true;
   };
+  // file name error
+  if (this.file_name != null) {
+    var extension = this.file_name.split('.').pop();
+    if (forbidden.has(extension)) {
+      add(true, "File name extension is illegal: '" + extension + "'.");
+    }
+  }
   // file name length error
   if (this.max_file_name_length != null && this.file_name != null && this.file_name.length > this.max_file_name_length) {
     add(true, "The background file name is too long: " + this.file_name.length + " > " + this.max_file_name_length + ".");
