@@ -14,9 +14,12 @@ var Glam2Input = function(container, options) {
   this.embed_name = this.container.querySelector("span.motif_embed input.name");
   // activate motif editor
   this._source_update();
+  this._file_update();
   // add event listeners
   // detect changes in the motif source selection
   this.source.addEventListener("change", function() { me._source_update(); }, false);
+  // detect changing of the uploaded file
+  this.file_input.addEventListener("change", function(e) { me._file_update(); }, false);
   // detect reset events
   if (this.source.form != null) {
     this.source.form.addEventListener("reset", function() { me.reset(); }, false);
@@ -53,6 +56,8 @@ Glam2Input.prototype.reset = function() {
     opt = this.source.options[i];
     opt.selected = opt.defaultSelected;
   }
+  this.file_input.value = "";
+  this._file_update();
   this._source_update();
 };
 
@@ -74,6 +79,18 @@ Glam2Input.prototype.get_alphabets = function() {
   return null;
 };
 
+Glam2Input.prototype._file_update = function() {
+  "use strict";
+   if (this.file_input.value != null) {
+     var extension = this.file_input.value.split('.').pop();
+     if (forbidden.has(extension)) {
+       alert("Motif file name extension is illegal: '" + extension + "'.");
+       this.file_input.value = "";
+       return false;
+     }
+  }
+};
+
 Glam2Input.prototype._source_update = function() {
   "use strict";
   var source, classes;
@@ -89,5 +106,6 @@ Glam2Input.prototype._source_update = function() {
     this.embed_name.disabled = false;
     this.file_input.disabled = true;
   }
-  
 };
+
+
