@@ -36,6 +36,21 @@ LociChecker.prototype.process_file = function(file_input) {
   comment_count = 0;
   if (this.give_up) return;
 
+  // file name error
+  if (file_input.name != null) {
+    var extension = file_input.name.split('.').pop();
+    if (forbidden.has(extension)) {
+      var error = {
+        name: "Loci BED file name extension is illegal",
+        type: FileType.INVALID_BED,
+        message: "'" + extension + "'."
+      }
+      this.handler.error_format(error);
+      this._signal_stop();
+      return;
+    }
+  }
+
   // file size error 
   if (this.handler.max_file_size != null && file_input.size > this.handler.max_file_size) {
     error = {
