@@ -68,6 +68,7 @@ function general_changed() {
   if (background != null && background.changed()) return true;
   if (!/^\s*6\s*$/.test($("minw").value)) return true;
   if (!/^\s*15\s*$/.test($("maxw").value)) return true;
+  if (! $("pgc").checked) return true;
   return false;
 }
 
@@ -77,6 +78,7 @@ function general_reset(evt) {
   $("dna2rna").checked = false;
   $("minw").value = 6;
   $("maxw").value = 15;
+  $("pgc").checked = true;
 }
 
 function meme_check(alphs) {
@@ -125,6 +127,7 @@ function streme_changed() {
   "use strict";
   if (!$("streme_enable_pthresh").checked) return true;
   if (!/^\s*0\.05\s*$/.test($("streme_pthresh").value)) return true;
+  if (!$("streme_align_center").checked) return true;
   return false;
 }
 
@@ -136,6 +139,7 @@ function streme_reset(evt) {
   $("streme_enable_nmotifs").checked = false;
   $("streme_nmotifs").value = 5;
   $("streme_nmotifs").disabled = true;
+  $("streme_align_center").checked = true;
 }
 
 function centrimo_check(alphs) {
