@@ -12,6 +12,7 @@ window.onload = function() {
   if (! num_psms || num_psms==0) {
     sessionStorage.setItem("num_psms", 1);
     sessionStorage.setItem("psm_1_format", "unknown");
+    sessionStorage.setItem("psm_1_sequence_length", "0");
     num_psms = 1;
   }
   for (i = 1; i <= num_psms; i++) {
@@ -42,6 +43,7 @@ function on_alg_change(partial) {
     ? 'block' : 'none');
   $("motifx_thresholds").style.display = ($('alg_mtfx').checked ? 'block' : 'none');
   $("modl_thresholds").style.display = ($('alg_modl').checked ? 'block' : 'none');
+//FIXME (TLB) I don't understand why this is needed or desireable.  Should be 5, always?
   if (partial != 1) {$("occurs").value = ($('alg_mtfx').checked ? 20 : 5);}
 }
 
@@ -56,7 +58,7 @@ function add_ptm() {
     return false;
   }
 
-  // Check that previous row has an uploaded PSM.
+  // Check that previous row has an uploaded PTM.
   var previousFile = $("psm_" + old_num);
   if (! previousFile.files[0]) {
     alert("Please select a file using the previous Browse button first.");
@@ -130,6 +132,61 @@ function trim_whitespace(string) {
   return string;
 } // trim_whitespace
 
+function check_prealigned_file(psm_name, filename, format, lines) {
+  var i;
+  const iupac = /^[ABCDEFGHIJKLMNPQRSTVWXYZ\*\.]*$/i;
+  var sequence_length = 0;
+  var expect_seqid = (format == "FASTA") ? true : false;
+  for (i=0; i<lines.length; i++) {
+    if (lines[i].match(/^\s*$/)) continue;	// Skip empty lines.
+    // Check and skip sequence ID lines if format is FASTA.
+    if (expect_seqid) {
+      if (! lines[i].match(/^>[^\s]+/)) {
+	alert("Expected sequence ID line in FASTA file at line " + (i+1) + 
+	  " but found': " + lines[i] + "'.");
+	return false;
+      } else {
+	expect_seqid = false;
+	continue;
+      }
+    } else {
+      expect_seqid = (format == "FASTA") ? true : false;
+    }
+    var sequence = lines[i].replace(/\s+/g, ''); 	// Remove whitespace.
+    // Check that only IUPAC characters are in the sequence.
+    if (! iupac.test(sequence)) {
+      alert("Illegal characters in " + format + " file '" + filename + "' line " + (i+1) +
+       ": '" + lines[i]) + "'";
+      return false;
+    }
+    // Check that all lines have the same length.
+    if (sequence_length > 0 && sequence.length != sequence_length) {
+      alert("All sequences in " + format + " file '" + filename + 
+	"' must have the same length as the first one: " + 
+	sequence_length + ".\nThe sequence in line " + (i+1) + " has length " + sequence.length + ".");
+      return false;
+    }
+    // Check that sequence length is odd.
+    if (sequence.length % 2 === 0) {
+      alert("The length of sequences in prealigned files must be odd. The sequence in line " + 
+        (i+1) + " has length " + sequence.length + ".");
+      return false;
+    }
+    sequence_length = sequence.length;
+  }
+  // Check that this file has the same sequence length as psm_1.
+  if (psm_name == "psm_1") {
+    sessionStorage.setItem("psm_1_sequence_length", sequence_length);
+  } else if (sequence_length != sessionStorage.getItem("psm_1_sequence_length")) {
+    alert("All sequences in " + format + " files must have the same length as in the first file: " + 
+      sessionStorage.getItem("psm_1_sequence_length") + ".\nFile "+ filename + " has sequence length " + 
+      sequence_length + ".");
+    return false;
+  }
+
+  return true;
+}
+
 function check_column_name(psm_column_name) {
 
   // Trim whitespace from the column name before checking.
@@ -138,6 +195,8 @@ function check_column_name(psm_column_name) {
   // Check that all the PTMs contain the named column (or are prealigned if column name is blank).
   var n_bad = 0;
   var bad_files = '';
+  var n_short_line_files = 0;
+  var short_line_files = '';
   var num_rows = sessionStorage.getItem("num_psms");
   for (i=1; i<=num_rows; i++) {
     var psm_name = "psm_" + i;
@@ -153,7 +212,7 @@ function check_column_name(psm_column_name) {
     if (psm_column_name == '') {				// name is blank
       var format = sessionStorage.getItem(psm_name + "_format");
       if (! (format == "FASTA" || format == "Raw")) {
-        if (i==1) {		// First file must be prealigned if column name is blank.
+        if (i==1) {		// First file must be FASTA or Raw if column name is blank.
 	 alert("You cannot specify a blank 'Modified Peptide Column Name' with '" + format + "' PTM files.");
 	 return false;
         } else {
@@ -171,25 +230,43 @@ function check_column_name(psm_column_name) {
       if (j == num_fields) {
 	bad_files = bad_files + '\n\t\t' + file_name;
 	n_bad++;
+      } else {
+	if (j > sessionStorage.getItem(psm_name + "_min_field")) {
+	  short_line_files = short_line_files + '\n\t\t' + file_name;
+	  n_short_line_files++;
+        }
       }
     }
   }
 
-  // Report any bad files.
+  // Report any files that are missing the sequence column or are the wrong format.
   if (n_bad > 0) {
     var file = (n_bad == 1) ? 'file' : 'files';
     var does = (n_bad == 1) ? 'does' : 'do';
     var is = (n_bad == 1) ? 'is' : 'are';
     if (psm_column_name != '') {
-      alert("The PTM " + file + ":" + bad_files + 
+      alert("The PSM " + file + ":" + bad_files + 
 	"\n" + does + " not contain a column named:\n\t\t" +
-	"'" + psm_column_name + "'" +
+	"'" + psm_column_name + "'." +
 	"\nPlease enter a new Modified Peptide Column Name or remove/change the PTM " + file + ".");
     } else {
       alert("The PTM " + file + ":" + bad_files + 
 	"\n" + is + " not in 'FASTA' or 'Raw' format but the first file is." +
 	"\nPlease enter a new first file or remove/change the listed PTM " + file + ".");
+      $("psm_column_name").style.display = 'none';
     }
+  }
+
+  // Report any files that have too few columns in some sequences.
+  if (n_short_line_files > 0) {
+    var file = (n_short_line_files == 1) ? 'file' : 'files';
+    var has = (n_short_line_files == 1) ? 'has' : 'have';
+    alert("The PSM " + file + ":" + short_line_files + 
+      "\n" + has + " one or more entries that do not contain the modified peptide column labelled '" +
+      psm_column_name + "'." + "\nPlease remove or change the PSM " + file + ".");
+  }
+
+  if (n_bad > 0 || n_short_line_files > 0) {
     return false;
   } else {
     return true;
@@ -237,49 +314,83 @@ function on_file_change(psm_name) {
   var f = $(psm_name).files[0];
 
   if (f) {
+
+    // Block certain filename extensions.
+    var extension = f.name.split('.').pop();
+    if (forbidden.has(extension)) {
+      alert("Illegal filename: '" + f.name + "'.\n" +
+        "You may not upload a file with extension '" + extension + "'.");
+      remove_ptm(psm_name);	// Remove PTM line and ones after it.
+      return false;
+    }
+    
     var r = new FileReader();
     r.onload = function(e) {
       var contents = e.target.result;
       var lines = contents.split('\n');
-      var line0 = '';
-      var line1 = '';
 
-      // Find the first non-empty line (header).
-      for (i=0; i<lines.length; i++) {
-        var tmp = lines[i].replace(/\s+/g, '');
-        if (tmp.length != 0) break;
-      }
-      if (i < lines.length) {		// in case file is empty
-        line0 = lines[i];
-      }
-
-      // Find the next non-empty line (fields).
-      for (++i; i<lines.length; i++) {
-        var tmp = lines[i].replace(/\s+/g, '');
-        if (tmp.length != 0) break;
-      }
-      if (i < lines.length) {		// in case file only has one line
-        line1 = lines[i];
+      // Find the first two non-empty lines (header and first entry)
+      // and save them in sessionStorage.
+      var idx_nonempty = 0;
+      for (i=0; i<lines.length && idx_nonempty < 2; i++) {
+        if (lines[i].match(/^\s*$/)) continue;	// Skip empty lines.
+        if (idx_nonempty == 0) {
+          sessionStorage.setItem(psm_name + "_line0", lines[i]);
+        } else {
+          sessionStorage.setItem(psm_name + "_line1", lines[i]);
+        }
+        idx_nonempty++;
       }
       
-      // Save the first two (non-blank) lines of the file in session storage.
-      sessionStorage.setItem(psm_name + "_line0", line0);
-      sessionStorage.setItem(psm_name + "_line1", line1);
-
       // Guess the file format.
       var filetype = guess_ptm_file_type(psm_name);
       sessionStorage.setItem(psm_name + "_format", filetype.format);
       $(psm_name + "_format").innerHTML = filetype.format;
-   
+
+      // Get the minimum number of fields in each line in PTM file.
+      // Check for illegal character combinations "<%" and "%>".
+      var max_field, min_field = 1e6;
+      for (i=0; i<lines.length; i++) {
+	if (lines[i].match(/^\s*$/)) continue;	// Skip empty lines.
+	// Disallow JSP.
+	if (lines[i].match(/<%|%>/)) {
+	  alert("Illegal File: The file:\n" + f.name +
+            "\ncontains the illegal character combinations '<%' and/or '%>'.");
+          remove_ptm(psm_name);	// Remove PTM line and ones after it.
+	  return false;
+	}
+	var matches = lines[i].match(/\t/g);
+        max_field = matches ? matches.length : 0;
+	if (max_field < min_field) min_field = max_field;
+      }
+
+      // Check FASTA and Raw syntax.
+      if (filetype.format == "Raw" || filetype.format == "FASTA") {
+        // Check that you can't mix prealigned and PSM files.
+        if (psm_name != "psm_1") {
+          var format1 = sessionStorage.getItem("psm_1_format");
+          if (format1 != "Raw" && format1 != "FASTA") {
+            alert("File '" + f.name + "' is a prealigned file but needs to be a PSM file.");
+            remove_ptm(psm_name);	// Remove PTM line and ones after it.
+	    return false;
+          }
+        }
+        if (! check_prealigned_file(psm_name, f.name, filetype.format, lines)) { 
+          remove_ptm(psm_name);	// Remove PTM line and ones after it.
+          return false;
+        }
+      } else {
+        // Save the index of the last field in the shortest entry.
+        sessionStorage.setItem(psm_name + "_min_field", min_field);
+      }
+ 
       // Set the probable modified peptide column name and update the
       // filter menu if this is the first PTM file.
       if (psm_name == "psm_1") { 
-        //if (filetype.format == "unknown PSM format" && filetype.psm_column_name == "") {
         if (filetype.format == "unknown PSM format") {
           // new first file of unknown PSM format.
 	  updateColumnName(filetype.format);	// Update the modified peptide column name
 	  $("psm_column_name").value = filetype.psm_column_name;
-	  //$("psm_column_name").value = "";
 	  updateFilterMenu();			// Update the filter menu.
         } else {
 	  if (check_column_name(filetype.psm_column_name)) {
@@ -295,7 +406,6 @@ function on_file_change(psm_name) {
         }
       } else {
         if (! check_column_name($("psm_column_name").value)) {
-          remove_ptm(true);			// Remove the PTM line.
           return false;
         }
       }
@@ -357,13 +467,14 @@ function updateFilterMenu() {
 
 } // updateFilterMenu
 
-// Clear all or just the last PTM input rows (if evt != null).
+// Clear given PTM row and all rows after it, or all rows if evt is null.
 function remove_ptm(evt) {
   "use strict";
   var num_rows = sessionStorage.getItem("num_psms");
   while (num_rows >= 1) {
     // Clear the filename and format, and turn off row display.
     $("psm_" + num_rows).value = $("psm_" + num_rows).defaultValue;
+    var psm_name = "psm_" + num_rows;
     var psm_format = "psm_" + num_rows + "_format";
     $(psm_format).innerHTML = '';
     sessionStorage.setItem(psm_format, '');
@@ -373,7 +484,7 @@ function remove_ptm(evt) {
       $("psm_row_" + num_rows).style.display = "none"
     }
     num_rows--;
-    if (evt) break;	// clear last row only
+    if (evt == psm_name) 	break;	// clear row psm_name and rows after it
   }
   if (num_rows < 1) num_rows = 1;
   sessionStorage.setItem("num_psms", num_rows);
@@ -417,6 +528,16 @@ function check() {
   if (sequences != null) {
     if (!sequences.check()) return false;
   }
+  // Check that prealigned sequences match the motif width.
+  var format = sessionStorage.getItem("psm_1_format");
+  if (format == "FASTA" || format == "Raw") {
+    var sequence_length = sessionStorage.getItem("psm_1_sequence_length");
+    if (sequence_length != $('width').value) {
+      alert("Please set the motif width (" + $('width').value + 
+        ") in Advanced options to the prealigned sequence length: " + sequence_length + ".");
+      return false;
+    }
+  }
   if ((! $('alg_mtfx').checked) && $('harvard').checked) {
     $('harvard').checked = false;
   }
@@ -485,6 +606,7 @@ function on_form_reset(evt) {
     remove_ptm(null);
     fix_reset();
     sessionStorage.clear();     // reset all storage
+    window.onload();		// makes sure num_psms = 1
   }, 50);
 }
 
