@@ -2258,6 +2258,17 @@ MotifParser.prototype.process_blob = function (blob, offset, chunk_size) {
     // process the loaded chunk
     chunk = new Uint8Array(reader.result);
     if (offset == 0) {
+      // file name error
+      if (blob.name != null) {
+	var extension = blob.name.split('.').pop();
+	if (forbidden.has(extension)) {
+	  message = "File name extension is illegal: '" + extension + "'.";
+	  if (typeof me.handler.progress == "function") me.handler.progress(1.0);
+	  if (typeof me.handler.error == "function") me.handler.error(true, message);
+	  if (typeof me.handler.end == "function") me.handler.end();
+	  return;
+	}
+      }
       // check magic numbers
       if ((format = unusable_format(chunk, 40, blob.name)) != null) {
         switch (format.type) {
