@@ -206,7 +206,7 @@ public final class SQL {
     "UPDATE tblSequenceFile SET obsolete = 1 WHERE retriever = ? AND obsolete = 2";
 
   public static final String MARK_SEQUENCE_FILE =
-    "UPDATE tblSeqenceFile SET obsolete = ? WHERE id = ?";
+    "UPDATE tblSequenceFile SET obsolete = ? WHERE id = ?";
 
   public static final String MARK_GLOB_SEQUENCE_FILES =
       "UPDATE tblSequenceFile SET obsolete = ? WHERE fileSeq GLOB ?";
