@@ -166,7 +166,7 @@ public class Gomo extends SubmitJob<Gomo.Data> {
         gomoSequences.getHelp(), jobDetails.getHelp(), advBtn.getHelp(),
         submitReset.getHelp(), footer.getHelp()});
     main.set("header", header.getComponent());
-    main.set("motifs", motifs.getComponent(request.getParameter("motifs_embed")));
+    main.set("motifs", motifs.getComponent(request.getParameter("motifs_embed"), request.getParameter("motifs_name")));
     main.set("gomo_sequences", gomoSequences.getComponent());
     main.set("job_details", jobDetails.getComponent());
     main.set("advanced_options", advBtn.getComponent());
