@@ -160,7 +160,7 @@ public class Meme extends SubmitJob<Meme.Data> {
     control = new ComponentSequences(context, tmplMain.getSubtemplate("control"));
     background = new ComponentBfile(context, tmplMain.getSubtemplate("bfile"));
     jobDetails = new ComponentJobDetails(cache);
-    advancedOptions = new ComponentAdvancedOptions(cache);
+    advancedOptions = new ComponentAdvancedOptions(cache, tmplMain.getSubtemplate("advanced_options"));
     submitReset = new ComponentSubmitReset(cache, jobTable.getCount(), jobTable.getDuration());
     footer = new ComponentFooter(cache, msp);
   }
