@@ -215,7 +215,7 @@ public class Spamo extends SubmitJob<Spamo.Data> {
         submitReset.getHelp(), footer.getHelp()});
     main.set("header", header.getComponent());
     main.set("sequences", sequences.getComponent());
-    main.set("primary", primary.getComponent(request.getParameter("motifs_embed")));
+    main.set("primary", primary.getComponent(request.getParameter("motifs_embed"), request.getParameter("motifs_name")));
     main.set("secondaries", secondaries.getComponent());
     main.set("bfile", background.getComponent());
     main.set("job_details", jobDetails.getComponent());
