@@ -151,7 +151,7 @@ public class Tgene extends SubmitJob<Tgene.Data> {
     loci = new ComponentLoci2(context, tmplMain.getSubtemplate("loci"));
     tgenePanel = new ComponentTgene(context, tmplMain.getSubtemplate("tgene_panel"));
     jobDetails = new ComponentJobDetails(cache);
-    advBtn = new ComponentAdvancedOptions(cache);
+    advBtn = new ComponentAdvancedOptions(cache, tmplMain.getSubtemplate("advanced_options"));
     submitReset = new ComponentSubmitReset(cache, jobTable.getCount(), jobTable.getDuration());
     footer = new ComponentFooter(cache, msp);
   }
