@@ -2,8 +2,6 @@ package au.edu.uq.imb.memesuite.servlet.util;
 
 import au.edu.uq.imb.memesuite.data.*;
 import au.edu.uq.imb.memesuite.db.*;
-//import au.edu.uq.imb.memesuite.io.bed.BedException;
-//import au.edu.uq.imb.memesuite.io.bed.BedParser;
 import au.edu.uq.imb.memesuite.template.HTMLSub;
 import au.edu.uq.imb.memesuite.template.HTMLSubGenerator;
 import au.edu.uq.imb.memesuite.template.HTMLTemplate;
@@ -33,6 +31,9 @@ public class ComponentLoci2 extends PageComponent {
   private HTMLTemplate tmplLoci;
   private String prefix;
   private String fieldName; // for feedback
+  private Integer maxFileSize;
+  private Integer maxFileNameLen;
+  private int maxNameLen;	// sequence name length
   private HTMLTemplate title;
   private HTMLTemplate subtitle;
   private boolean enableNoLoci;
@@ -54,8 +55,14 @@ public class ComponentLoci2 extends PageComponent {
     this.context = context;
     HTMLTemplateCache cache = (HTMLTemplateCache)context.getAttribute(CACHE_KEY);
     tmplLoci = cache.loadAndCache("/WEB-INF/templates/component_loci.tmpl");
+    // getText and getInfo values can be OVERRIDDEN in the servlet template (e.g., tgene.tmpl)
+    // by placing tags inside the <!--{loci}--> subtemplate such as;
+    //       <!--{max_file_size}-->1000<!--{/max_file_size}-->
     prefix = getText(info, "prefix", "loci");
-    fieldName = getText(info, "description", "loci");
+    fieldName = getText(info, "description", "BED file of loci");
+    maxFileSize = getInt(info, "max_file_size", 80000000);
+    maxFileNameLen = getInt(info, "max_file_name_len", 100);
+    maxNameLen = getInt(info, "max_name_len", 50);
     registerFn = getText(info, "register", "nop");
     title = getTemplate(info, "title", null);
     subtitle = getTemplate(info, "subtitle", null);
@@ -71,14 +78,19 @@ public class ComponentLoci2 extends PageComponent {
 
     StringWriter buf = new StringWriter();
     JsonWr jsonWr = new JsonWr(buf, 18);
+    // Set up the options to be passed to the controller in the template.
     try {
       jsonWr.start();
       jsonWr.property("field", fieldName);
+      jsonWr.property("max_file_size", maxFileSize);
+      jsonWr.property("max_file_name_len", maxFileNameLen);
+      jsonWr.property("max_name_len", maxNameLen);
       jsonWr.end();
     } catch (IOException e) {
       // no IO exceptions should occur as this uses a StringBuffer
       throw new Error(e);
     }
+    loci.set("options", buf.toString());
     loci.set("register_component", registerFn);
     return loci;
   }
@@ -141,7 +153,7 @@ public class ComponentLoci2 extends PageComponent {
         file.deleteOnExit();
         out = new BufferedOutputStream(new FileOutputStream(file));
         //BedParser.parse(in, handler);
-	// Just copy the input BED file for now;  TODO
+	// Just copy the input BED file for now; TODO
         int numBytes;
 	      while ((numBytes = in.read(buffer)) != -1) {
           out.write(buffer, 0, numBytes);
