@@ -805,7 +805,17 @@ public class MotifValidator {
     nextTag(cursor, feedback); cursor.require(START_ELEMENT, null, "niter"); cursor.getElementText(); cursor.require(END_ELEMENT, null, "niter");
     nextTag(cursor, feedback); cursor.require(START_ELEMENT, null, "patience"); cursor.getElementText(); cursor.require(END_ELEMENT, null, "patience");
     nextTag(cursor, feedback); cursor.require(START_ELEMENT, null, "seed"); cursor.getElementText(); cursor.require(END_ELEMENT, null, "seed");
-    nextTag(cursor, feedback); cursor.require(START_ELEMENT, null, "useer"); cursor.getElementText(); cursor.require(END_ELEMENT, null, "useer");
+    nextTag(cursor, feedback); 
+    try {
+      // post 5.5.3 
+      cursor.require(START_ELEMENT, null, "notrim");
+      cursor.getElementText();
+      cursor.require(END_ELEMENT, null, "notrim");
+      nextTag(cursor, feedback); 
+    } catch (XMLStreamException e) {
+      // pre 5.5.3
+    } 
+    cursor.require(START_ELEMENT, null, "useer"); cursor.getElementText(); cursor.require(END_ELEMENT, null, "useer");
     nextTag(cursor, feedback); cursor.require(START_ELEMENT, null, "minscore"); cursor.getElementText(); cursor.require(END_ELEMENT, null, "minscore");
     nextTag(cursor, feedback); cursor.require(START_ELEMENT, null, "ignore_depth"); cursor.getElementText(); cursor.require(END_ELEMENT, null, "ignore_depth");
     nextTag(cursor, feedback); cursor.require(START_ELEMENT, null, "nsubsets"); cursor.getElementText(); cursor.require(END_ELEMENT, null, "nsubsets");
@@ -951,14 +961,14 @@ public class MotifValidator {
         cursor.nextNumberProperty("primary_positions");
         if (cursor.isProperty("control_source")) {	// optional
         cursor.nextStringProperty("control_source");
-        cursor.nextNumberProperty("control_count");
-        cursor.nextNumberProperty("control_positions");
+	  cursor.nextNumberProperty("control_count");
+	  cursor.nextNumberProperty("control_positions");
         }
       }
       if (cursor.isProperty("psp_source")) cursor.nextStringProperty("psp_source");
       cursor.nextListProperty("freqs", Number.class);
-      if (cursor.isProperty("sequences")) { // start sequences
-        cursor.next("sequences"); // start sequences
+      if (cursor.isProperty("sequences")) {		// start sequences
+        cursor.next("sequences");			// start sequences
 	cursor.next(JSON_START_LIST);
 	while (cursor.getEventType() != JSON_END_LIST) {
 	  cursor.next(JSON_START_OBJECT);
@@ -968,10 +978,10 @@ public class MotifValidator {
 	  cursor.next(JSON_END_OBJECT);
 	}
 	cursor.next(JSON_END_LIST);
-	cursor.next(JSON_END_PROPERTY); // end sequences
+	cursor.next(JSON_END_PROPERTY);			// end sequences
       }
       cursor.next(JSON_END_OBJECT);
-      cursor.next(JSON_END_PROPERTY); // end sequence_db
+      cursor.next(JSON_END_PROPERTY);			// end sequence_db
     } else if (isDreme) {
       cursor.skipProperty("sequence_db");
       cursor.skipProperty("control_db");
@@ -983,7 +993,7 @@ public class MotifValidator {
       cursor.skipProperty("sequence_db");
     }
 
-    cursor.next("motifs"); // start motifs
+    cursor.next("motifs");				// start motifs
     cursor.next(JSON_START_LIST);
     while (cursor.getEventType() != JSON_END_LIST) {
       cursor.next(JSON_START_OBJECT);
@@ -1014,6 +1024,9 @@ public class MotifValidator {
         cursor.nextNumberProperty("initial_width");
         cursor.nextStringProperty("seed");
         cursor.nextNumberProperty("score_threshold");
+        if (cursor.isProperty("npassing")) {		// post 5.5.3
+          cursor.skipProperty("npassing");
+        }
         cursor.nextNumberProperty("train_pos_count");
         cursor.nextNumberProperty("train_neg_count");
         cursor.nextNumberProperty("train_log_pvalue");
