@@ -36,7 +36,8 @@ public abstract class AbstractFtpSource implements Source, FtpSource {
   }
 
   public String getLocalName() {
-    return getNamePrefix() + getRemoteExt();
+    // Use the remote name as the local name to avoid collisions.
+    return getRemoteName();
   }
 
   @Override
