@@ -96,7 +96,7 @@ public class FtpFileListUpdater extends SequenceUpdater {
 	URL url = new URL(host + path + name);
 	URLConnection connection = url.openConnection();
 	long lastModified = connection.getLastModified();
-	long sourceSize = connection.getContentLength();
+	long sourceSize = connection.getContentLengthLong();
 	Calendar timestamp = Calendar.getInstance();
 	timestamp.setTimeInMillis(lastModified);
 	CsvSource httpSrc = new CsvSource(entry, alphabet, host, path, name, sourceSize, timestamp);
@@ -117,7 +117,7 @@ public class FtpFileListUpdater extends SequenceUpdater {
           }
 	}
       } catch (IOException e) {
-       logger.log(Level.SEVERE, "URL exception.");
+       logger.log(Level.SEVERE, "URL exception: " + e.getMessage());
       }
 
     } else {
@@ -151,8 +151,6 @@ public class FtpFileListUpdater extends SequenceUpdater {
 	    }
           }
 	}
-	ftp.logout();
-	ftp.disconnect();
       } catch (IOException e) {
 	logger.log(Level.SEVERE, "Skipping source: " + source, e);
       } finally {
@@ -274,7 +272,7 @@ public class FtpFileListUpdater extends SequenceUpdater {
         }
         reader = new SeqSourceReader(csv);
         while ((entry = reader.next()) != null) {
-          logger.log(Level.INFO, "Downloading entry " + entry.name);
+          logger.log(Level.INFO, "Downloading entry " + entry.name + " version " + entry.version);
           if (entry.has_dna) {
             navailable_genomes++;
             nhave_genomes += downloadSource(entry, AlphStd.DNA, dbTarget);
@@ -470,6 +468,23 @@ public class FtpFileListUpdater extends SequenceUpdater {
       return size;
     }
 
+    @Override
+    public String getRemoteExt() {
+      String ext;
+      ext = super.getRemoteExt();
+      // Avoid collisions if DNA and Protein remote files have the same extension.
+      if (ext.equals(".fa.gz") || ext.equals("fasta.gz")) {
+	if (alphabet.name().equals("DNA")) {
+	  ext = ".fna.gz";
+        } else if (alphabet.name().equals("PROTEIN")) {
+          ext = ".faa.gz";
+        } else if (alphabet.name().equals("RNA")) {
+          ext = ".fra.gz";
+        }
+      }
+      return ext;
+    }
+
     @Override
     public void setSourceFile(File file) {
       this.downloadedFile = file;
@@ -501,11 +516,14 @@ public class FtpFileListUpdater extends SequenceUpdater {
 
     @Override
     public long getSequenceEdition() {
-      //return timestamp.getTimeInMillis();
-      Date date = new Date(System.currentTimeMillis());
+      // Use the timestamp of the file as the sequenced edition (and version).
+      // Time stamp is a long with format: YYYYMMDD.
+      // Thus, if the DNA and protein files have different dates (at that resolution)
+      // they will be different versions.
+      Date date = new Date(timestamp.getTimeInMillis());
       Calendar calendar = Calendar.getInstance();
       calendar.setTime(date);
-      long edition = calendar.get(Calendar.DATE) + (100 * calendar.get(Calendar.MONTH)) + (100 * 100 * calendar.get(Calendar.YEAR));
+      long edition = calendar.get(Calendar.DATE) + (100 * (calendar.get(Calendar.MONTH)+1)) + (100 * 100 * calendar.get(Calendar.YEAR));
       System.out.println(entry.name + " (" + this.alphabet + ") edition: " + edition);
       return(edition);
     }
@@ -514,7 +532,7 @@ public class FtpFileListUpdater extends SequenceUpdater {
     public String getSequenceVersion() {
       String version;
       if (entry.version.equals("")) {
-	version = DateFormat.getDateInstance(DateFormat.MEDIUM).format(timestamp.getTime());
+        version = DateFormat.getDateInstance(DateFormat.MEDIUM).format(timestamp.getTime());
       } else {
         version = entry.version;
       }
@@ -540,17 +558,17 @@ public class FtpFileListUpdater extends SequenceUpdater {
 
     @Override
     public String getNamePrefix() {
-      String Version;
-      if (entry.version.equals("")) {
-        // Use date as version.
-	DateFormat simple = new SimpleDateFormat("dd-MM-yyyy");
-	Date lastModifiedDate = new Date(timestamp.getTimeInMillis());
-        Version = simple.format(lastModifiedDate); 
-      } else {
-        Version = entry.version;
+      String namePrefix = entry.root;
+      // Append version if there is one.
+      if (! entry.version.equals("")) {
+        namePrefix += "_" + entry.version ;
       }
-      //return entry.root + "_" + alphabet.name() + "_" + Version;
-      return entry.root + "_" + Version;
+      // Append timestamp date to make file names unique.
+      DateFormat simple = new SimpleDateFormat("dd-MM-yyyy");
+      Date lastModifiedDate = new Date(timestamp.getTimeInMillis());
+      namePrefix += "_" + simple.format(lastModifiedDate); 
+
+      return namePrefix;
     }
 
     @Override
