@@ -143,6 +143,7 @@ public abstract class SequenceUpdater implements Callable<Void> {
     Boolean obsolete = null;
     Connection conn = null;
     dbLock.readLock().lock();
+    Boolean readlock_set = true;
     try {
       conn = dataSource.getConnection();
       PreparedStatement ps = conn.prepareStatement(
@@ -161,12 +162,14 @@ public abstract class SequenceUpdater implements Callable<Void> {
       ps.close();
       if (deobsolete && exists && obsolete) {
 	//TLB: added next line to prevent deadlock
+        readlock_set = false;
         dbLock.readLock().unlock();
         dbLock.writeLock().lock();
         try {
           ps = conn.prepareStatement(MARK_SEQUENCE_FILE);
           ps.setInt(1, 0); // set obsolete = 0
           ps.setLong(2, fileId);
+          ps.executeUpdate();
           ps.close();
         } finally {
            dbLock.writeLock().unlock();
@@ -182,7 +185,9 @@ public abstract class SequenceUpdater implements Callable<Void> {
           logger.log(Level.SEVERE, "Unable to close database connection after error.");
         }
       }
-      dbLock.readLock().unlock();
+      if (readlock_set) { 
+        dbLock.readLock().unlock(); 
+      }
     }
     return exists;
   }
@@ -344,8 +349,9 @@ public abstract class SequenceUpdater implements Callable<Void> {
       return false;
     }
     // check the file size
-    if (target.length() != source.getRemoteSize()) {
-      logger.log(Level.WARNING, "Transfered file size does not match remote (" + source + ").");
+    // Ignore -1 remote size since the server may not be offering that field.
+    if (target.length() != source.getRemoteSize() && source.getRemoteSize() != -1) {
+      logger.log(Level.WARNING, "Transfered file size " + target.length() + " does not match remote size " + source.getRemoteSize() + " (" + source + ").");
       return false;
     }
     logger.log(Level.INFO, "Download of " + source + " complete.");
@@ -416,8 +422,9 @@ public abstract class SequenceUpdater implements Callable<Void> {
       return false;
     }
     // check the file size
-    if (target.length() != source.getRemoteSize()) {
-      logger.log(Level.WARNING, "Transfered file size does not match remote (" + source + ").");
+    // Ignore -1 remote size since the server may not be offering that field.
+    if (target.length() != source.getRemoteSize() && source.getRemoteSize() != -1) {
+      logger.log(Level.WARNING, "Transfered file size " + target.length() + " does not match remote size " + source.getRemoteSize() + " (" + source + ").");
       return false;
     }
     logger.log(Level.INFO, "Download of " + source + " complete.");
