@@ -24,6 +24,7 @@ public class JsonWr {
   private JsonState state;
   private Deque<JsonState> stack;
   private StringWriter line;
+  private Boolean lastElement;
 
   private static final Pattern numberPattern = Pattern.compile(
       "^-?(?:0|[1-9]\\d*)(?:\\.\\d+)?(?:[eE][+-]?\\d+)?$");
@@ -65,6 +66,7 @@ public class JsonWr {
     this.state = JsonState.READY;
     this.stack = new ArrayDeque<JsonState>();
     this.line = new StringWriter(lineCols);
+    this.lastElement = false;
   }
 
   /**
@@ -537,9 +539,16 @@ public class JsonWr {
    */
   public void array(String[] values) throws IOException {
     startArray();
+    int size = values.length;
+    int count = 0;
     for (String value : values) {
+      if (count == (size - 1)) {
+        lastElement = true;
+      }
       value(value);
+      ++count;
     }
+    lastElement = false;
     endArray();
   }
 
@@ -550,9 +559,16 @@ public class JsonWr {
    */
   public void array(long[] values) throws IOException {
     startArray();
+    int size = values.length;
+    int count = 0;
     for (long value : values) {
+      if (count == (size - 1)) {
+        lastElement = true;
+      }
       value(value);
+      ++count;
     }
+    lastElement = false;
     endArray();
   }
 
@@ -563,9 +579,16 @@ public class JsonWr {
    */
   public void array(double[] values) throws IOException {
     startArray();
+    int size = values.length;
+    int count = 0;
     for (double value : values) {
+      if (count == (size - 1)) {
+        lastElement = true;
+      }
       value(value);
+      ++count;
     }
+    lastElement = false;
     endArray();
   }
 
@@ -576,9 +599,16 @@ public class JsonWr {
    */
   public void array(boolean[] values) throws IOException {
     startArray();
+    int size = values.length;
+    int count = 0;
     for (boolean value : values) {
+      if (count == (size - 1)) {
+        lastElement = true;
+      }
       value(value);
+      ++count;
     }
+    lastElement = false;
     endArray();
   }
 
@@ -589,9 +619,16 @@ public class JsonWr {
    */
   public void array(JsonValue[] values) throws IOException {
     startArray();
+    int size = values.length;
+    int count = 0;
     for (JsonValue value : values) {
+      if (count == (size - 1)) {
+        lastElement = true;
+      }
       value(value);
+      ++count;
     }
+    lastElement = false;
     endArray();
   }
 
